using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Algorithms.Utils;

namespace MyMediaProject.RecommenderSystem.Algorithms
{
    /// <summary>
    /// Abstract class for matrix factorization based item predictors
    /// </summary>
    /// <author>Steffen Rendle, University of Hildesheim</author>   
    public abstract class ItemRecommenderMF : ItemRecommenderMemory
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ItemRecommenderMF"/> class.
        /// </summary>
        /// <param name="target_relation">The target_relation.</param>
        public ItemRecommenderMF(RelationType target_relation) : base(target_relation) { }

        /// <summary>
        /// User feature matrix
        /// </summary>
        protected Matrix<double> user_feature;
        /// <summary>
        /// Item feature matrix
        /// </summary>
        protected Matrix<double> item_feature;

        /// <summary>
        /// Number of features
        /// </summary>
        public int num_feature = 10;

        /// <inheritdoc />
        protected override void _Train()
        {
            base._Train();
            user_feature = new Matrix<double>(max_user_id + 1, num_feature);
            item_feature = new Matrix<double>(max_item_id + 1, num_feature);
        }

        /// <inheritdoc />
        public override double PredictWeight(int user_id, int item_id)
        {
            if ((user_id < 0) || (user_id >= user_feature.dim1))
            {
                throw new Exception("user is unknown");
            }
            if ((item_id < 0) || (item_id >= item_feature.dim1))
            {
                throw new Exception("item is unknown");
            }
            double result = 0;
            for (int f = 0; f < num_feature; f++)
            {
                result += user_feature.get(user_id, f) * item_feature.get(item_id, f);
            }
            return result;
        }

    }

}