using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Algorithms.Utils;

namespace MyMediaProject.RecommenderSystem.Algorithms
{
    /// <summary>
    /// Abstract item recommender class, that loads the training data into two sparse matrices: one column-wise and one row-wise
    /// </summary>
    /// <author>Steffen Rendle, University of Hildesheim</author>
    public abstract class ItemRecommenderMemory : ItemRecommender
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ItemRecommenderMemory"/> class.
        /// </summary>
        /// <param name="target_relation">The target_relation.</param>
        public ItemRecommenderMemory(RelationType target_relation) : base(target_relation) { }


        /// <summary>
        /// Max user id
        /// </summary>
        protected int max_user_id = 0;
        /// <summary>
        /// Max item id
        /// </summary>
        protected int max_item_id = 0;
        /// <summary>
        /// Data user
        /// </summary>
        protected SparseBooleanMatrix data_user;
        /// <summary>
        /// Data item
        /// </summary>
        protected SparseBooleanMatrix data_item;

        /// <inheritdoc />
        protected override void _Train()
        {
            // (1) read the data in two sparse matrices, one is user-wise, the other item-wise
            data_user = new SparseBooleanMatrix();
            data_item = new SparseBooleanMatrix();
            IEntityRelationDataReader data_reader = backend.GetRelation(target_relation);
            while (data_reader.Read())
            {
                int user_id = data_reader.GetInt32(0);
                int item_id = data_reader.GetInt32(1);
                data_user.addEntry(user_id, item_id);
                data_item.addEntry(item_id, user_id);
                max_user_id = Math.Max(max_user_id, user_id);
                max_item_id = Math.Max(max_item_id, item_id);
            }
        }

    }

}