using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm;
using MyMediaProject.RecommenderSystem.Core;

namespace MyMediaProject.RecommenderSystem.Algorithms
{
    /// <summary>
    /// Most-popular item recommender. Items are weighted by how often they have been seen in the past.
    /// This method is unpersonalized.
    /// </summary>
    /// <author>Steffen Rendle, University of Hildesheim</author>
    public class ItemRecommenderMostPopular : ItemRecommender
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ItemRecommenderMostPopular"/> class.
        /// </summary>
        /// <param name="target_relation">The target_relation.</param>
        public ItemRecommenderMostPopular(RelationType target_relation) : base(target_relation) { }
        /// <summary>
        /// View count
        /// </summary>
        protected Dictionary<int, int> view_count = new Dictionary<int, int>();

        /// <inheritdoc />
        protected override void _Train()
        {
            IEntityRelationDataReader data = backend.GetRelation(target_relation);
            while (data.Read())
            {
                int item_id = data.GetInt32(1);
                if (!view_count.ContainsKey(item_id))
                {
                    view_count[item_id] = 1;
                }
                else
                {
                    view_count[item_id]++;
                }
            }
        }
        /// <inheritdoc />
        public override double PredictWeight(int user_id, int item_id)
        {
            int cnt = 0;
            if (view_count.TryGetValue(item_id, out cnt))
            {
                return cnt;
            }
            else
            {
                return 0;// Double.NegativeInfinity;
            }
        }
       
    }

}