using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Algorithms.Utils;

namespace MyMediaProject.RecommenderSystem.Algorithms
{
    /// <summary>
    /// k-nearest neighbor collaborative filtering using cosine-similarity
    /// k=\infty
    /// </summary>
    /// <author>Steffen Rendle, University of Hildesheim</author>
    public class ItemRecommenderkNN : ItemRecommenderMemory
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ItemRecommenderkNN"/> class.
        /// </summary>
        /// <param name="target_relation">The target_relation.</param>
        public ItemRecommenderkNN(RelationType target_relation) : base(target_relation) { }

        /// <summary>
        /// Correlation
        /// </summary>
        protected Matrix<double> correlation;

        /// <inheritdoc />
        protected override void _Train()
        {
            base._Train();
         
            int num_item = max_item_id + 1;
		    correlation = new Matrix<double>(num_item, num_item);
            correlation.init(0);

            for (int i = 0; i < num_item; i++) {
				correlation.set(i, i, 1);
                if (! data_item.ContainsKey(i))
                {
                    continue;
                }
			    HashSet<int> users_i = data_item[i];
								
				for (int j = 0; j < num_item; j++) {
                    int cntr = 0;
					// cntr = | S_u && S_i |
                    if (data_item.ContainsKey(j))
                    {
                        foreach (int user_id in data_item[j])
                        {
                            if (users_i.Contains(user_id))
                            {
                                cntr++;
                            }
                        }
                        correlation.set(i, j, (double)cntr / (Math.Sqrt(users_i.Count) * Math.Sqrt(data_item[j].Count)));
                    }
                    else
                    {
                        correlation.set(i, j, 0);
                    }
				}
			}
			
			for (int i = 0; i < num_item; i++) {
				for (int j = 0; j < i; j++) {
					if (correlation.get(i,j) != correlation.get(j,i)) {
                        throw new Exception("implementation is wrong.");
                    }
                    //correlation[i][j] = correlation[j][i];					
				}
			} 
        }


        /// <inheritdoc />
        public override double PredictWeight(int user_id, int item_id)
        {
            if ((user_id < 0) || (user_id > max_user_id))
            {
                throw new Exception("user is unknown");
            }
            if ((item_id < 0) || (item_id > max_item_id))
            {
                throw new Exception("item is unknown");
            }
            double result = 0;
            foreach (int item_id2 in data_user[user_id]) {
                result += correlation.get(item_id, item_id2);
            }
            return result;
        }
    }

}