using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm;
using MyMediaProject.RecommenderSystem.Core;

namespace MyMediaProject.RecommenderSystem.Algorithms
{
    /// <summary>
    /// Computes the average rating value over all ratings.
    /// </summary>
    /// <author>Steffen Rendle, University of Hildesheim</author>
    public class RatingPredictorGlobalAverage : RatingPredictor
    {
        /// <summary>
        /// Global average.
        /// </summary>
        public double global_average;

        /// <inheritdoc />
        protected override void _Train()
        {
            IEntityRelationDataReader rating = backend.GetRelation(RelationType.Rated);
            int cnt = 0;
            double sum = 0;
            while (rating.Read())
            {
                cnt++;
                sum += rating.GetDouble(2);
            }
            global_average = (double)sum / cnt;
        }

        /// <inheritdoc />
        public override double PredictRating(int user_id, int item_id)
        {
            return global_average;
        }
    }

    
    /// <summary>
    /// Computes the average rating value for a specific user.
    /// </summary>
    /// <author>Steffen Rendle, University of Hildesheim</author>
    public class RatingPredictorUserAverage : RatingPredictor
    {
        /// <summary>
        /// Global average
        /// </summary>
        public double global_average;
        /// <summary>
        /// Average
        /// </summary>
        protected class Average
        {
            /// <summary>
            /// Count
            /// </summary>
            public int cnt = 0;
            /// <summary>
            /// Sum
            /// </summary>
            public double sum = 0;
            /// <summary>
            /// Averages this instance.
            /// </summary>
            /// <returns></returns>
            public double average() { return (double)sum / cnt; }
        }

        /// <summary>
        /// List of average statistics
        /// </summary>
        protected List<Average> statistics = new List<Average>();

        /// <inheritdoc />
        protected override void _Train()
        {
            IEntityRelationDataReader rating = backend.GetRelation(RelationType.Rated);

            double sum = 0;
            int cnt = 0;

            statistics.Clear();
            while (rating.Read())
            {
                int index = rating.GetInt32(0);
                while (index >= statistics.Count)
                {
                    statistics.Add(new Average());
                }
                statistics[index].cnt++;
                statistics[index].sum += rating.GetDouble(2);
                sum += rating.GetDouble(2);
                cnt++;
            }
            global_average = sum / cnt;
        }

        /// <inheritdoc />
        public override double PredictRating(int user_id, int item_id)
        {
            if (user_id < statistics.Count)
            {
                if (statistics[user_id].cnt == 0)
                {
                    return global_average;
                }
                return statistics[user_id].average();
            }
            else
            {
                throw new Exception("unknown user");
            }
        }
    }

    /// <summary>
    /// Computes the average rating value for a specific item.
    /// </summary>
    /// <author>Steffen Rendle, University of Hildesheim</author>
    public class RatingPredictorItemAverage : RatingPredictor
    {
        /// <summary>
        /// Global average
        /// </summary>
        public double global_average;
        /// <summary>
        /// Average
        /// </summary>
        protected class Average
        {
            /// <summary>
            /// count
            /// </summary>
            public int cnt = 0;
            /// <summary>
            /// Sum
            /// </summary>
            public double sum = 0;
            /// <summary>
            /// Averages this instance.
            /// </summary>
            /// <returns></returns>
            public double average() { return (double)sum / cnt; }
        }

        /// <summary>
        /// List of averages
        /// </summary>
        protected List<Average> statistics = new List<Average>();

        /// <inheritdoc />
        protected override void _Train()
        {
            IEntityRelationDataReader rating = backend.GetRelation(RelationType.Rated);

            double sum = 0;
            int cnt = 0;

            statistics.Clear();
            while (rating.Read())
            {
                int index = rating.GetInt32(1);
                while (index >= statistics.Count)
                {
                    statistics.Add(new Average());
                }
                statistics[index].cnt++;
                statistics[index].sum += rating.GetDouble(2);
                sum += rating.GetDouble(2);
                cnt++;
            }
            global_average = sum / cnt;
        }

        /// <inheritdoc />
        public override double PredictRating(int user_id, int item_id)
        {
            if (item_id < statistics.Count)
            {
                if (statistics[item_id].cnt == 0)
                {
                    return global_average;
                }
                return statistics[item_id].average();
            }
            else
            {
                throw new Exception("unknown item");
            }
        }
    }



}
