﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Algorithms.Utils
{
    /// <summary>
    /// Drawing random values from a normal distibuted using a simple rejection method.
    /// </summary>
    /// <author>Steffen Rendle, University of Hildesheim</author>
    public class Random : System.Random
    {
        private static Random instance = null;

        /// <summary>
        /// Gets the instance.
        /// </summary>
        /// <returns></returns>
        public static Random getInstance()
        {
            if (instance == null)
            {
                instance = new Random();
            }
            return instance;
        }

        private double sqrt_e_div_2_pi = Math.Sqrt(Math.E / (2 * Math.PI));
        /// <summary>
        /// Nexts the exp.
        /// </summary>
        /// <param name="lambda">The lambda.</param>
        /// <returns></returns>
        public double NextExp(double lambda)
        {
            double u = this.NextDouble();
            return -(1 / lambda) * Math.Log(1 - u, Math.E);
        }

        /// <summary>
        /// Nexts the normal.
        /// </summary>
        /// <returns></returns>
        public double NextNormal()
        {
            double y;
            double x;
            do 
            {
                double u = this.NextDouble();
                x = this.NextExp(1);
                y = 2*u*sqrt_e_div_2_pi*Math.Exp(-x);
            } while (y< (2/(2*Math.PI))*Math.Exp(-0.5*x*x));
            if (this.NextDouble() < 0.5)
            {
                return x;
            }
            else
            {
                return -x;
            }
        }
        /// <summary>
        /// Nexts the normal.
        /// </summary>
        /// <param name="mean">The mean.</param>
        /// <param name="stdev">The stdev.</param>
        /// <returns></returns>
        public double NextNormal(double mean, double stdev)
        {
            return mean + stdev * NextNormal();
        }
        
    }
}
