﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm;
using MyMediaProject.RecommenderSystem.Core;

namespace MyMediaProject.RecommenderSystem.Algorithms
{
    /// <summary>Basic virtual profile based recommender:
    /// r(i,v) := \avg_{ (i',v',u',r') \in S} I(i=i' &amp;&amp; v=v') * r 
    /// </summary>
    /// <author>Steffen Rendle, University of Hildesheim</author>
    public class VirtualProfileRecommender : IEntityRelationPredictor
    {
        /// <inheritdoc />
        public int[] PredictEntity(EntityType entityType, int[] keys, int count)
        {
            throw new Exception("not yet implemented");
        }
        /// <inheritdoc />
        public double PredictRelation(RelationType relationType, int[] keys)
        {
            if (relationType != RelationType.VirtualProfileRated)
            {
                throw new Exception("type is not social network");
            }
            if (keys.Length == 3)
            {
                return PredictRating(keys[0], keys[1], keys[2]);
            }
            else
            {
                return PredictRating(keys[0], keys[1], -1);
            }
        }

        private IEntityRelationDataProvider backend;
        /// <summary>
        /// Initializes a new instance of the <see cref="VirtualProfileRecommender"/> class.
        /// </summary>
        /// <param name="backend">The backend.</param>
        public VirtualProfileRecommender(IEntityRelationDataProvider backend)
        {
            this.backend = backend;
        }

        /// <inheritdoc />
        protected double PredictRating(int item_id, int virtual_profile_id, int user_id)
        {
            IEntityRelationDataReader vp = backend.GetRelation(RelationType.VirtualProfileRated);
            int cnt = 0;
            double sum = 0;
            int item_cnt = 0;
            double item_sum = 0;
            int profile_cnt = 0;
            double profile_sum = 0;
            int global_cnt = 0;
            double global_sum = 0;
            while (vp.Read())
            {
                int triple_item_id = vp.GetInt32(0);
                int triple_vp_id = vp.GetInt32(2);
                double triple_rating = vp.GetDouble(1);
                if (triple_item_id == item_id) {
                    item_cnt++;
                    item_sum += triple_rating;
                    if (triple_vp_id == virtual_profile_id)
                    {
                        cnt++;
                        sum += triple_rating;
                    }
                };
                if (triple_vp_id == virtual_profile_id)
                {
                    profile_cnt++;
                    profile_sum += triple_rating;
                }
                global_cnt++;
                global_sum += triple_rating;
            }
            if (cnt > 0)
            {
                return sum / cnt;
            } else if ((item_cnt > 0) || (profile_cnt > 0)) {
                return (item_sum+profile_sum) / (profile_cnt+item_cnt);
            } else {
                return global_sum / global_cnt;
            }
        }
    }
}
