﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EMIC.Design;

namespace MyMediaProject.RecommenderSystem.Core.Config
{
    /// <summary>
    /// Holds data of an individual configurational property
    /// </summary>
    public class ConfigProperty
    {
        /// <summary>
        /// Is set to true, if the initial data member is changed via one of the 'set'-methods
        /// </summary>
        public Boolean IsChanged
        {
            get 
            {
                if (String.Compare(Value, DefaultValue, StringComparison.Ordinal) == 0)
                {
                    return false;
                }
                return (String.Compare(Value, DefaultValue, StringComparison.Ordinal) != 0);
            }
        }

        /// <summary>
        /// Unique name that identifies the property
        /// </summary>
        public String Name { get; set; }

        /// <summary>
        /// Textual value of the property
        /// </summary>
        public String Value { get; set; }

        /// <summary>
        /// Human friendly description of the property. Will be displayed to admin user when configuring.
        /// </summary>
        public String Description { get; set; }

        /// <summary>
        /// The value that has been set by the property creator initially. Can be used as a reference or considered as a 'smart default'.
        /// This value is set to the repository version of property's value.
        /// </summary>
        public String DefaultValue { get; set; }

        /// <summary>
        /// Private value that is designed to be programmatically generated, set and handled (and not modified manually)
        /// </summary>
        public String PrivateValue { get; set; }

        /// <summary>
        /// ConfigProperty Constructor
        /// </summary>
        /// <param name="name"></param>
        /// <param name="value"></param>
        /// <param name="description"></param>
        public ConfigProperty(String name, String value, String description)
        {
            this.Name = name;
            this.Value = value;
            this.Description = description;

            //TraceLog.Write(LogEventType.Information, Property: '{0}' = '{1}' initialized.", new object[]{name, value});
        }

        /// <summary>
        /// Lists property's details for tracing purposes
        /// </summary>
        /// <returns></returns>
        public override String ToString()
        {
            String breakLine = "\r\n";
            return "Property '" + Name + "':" + breakLine
                    + "\t Name : '" + Name + "'," + breakLine
                    + "\t Value : '" + Value + "'," + breakLine
                    + "\t Description : '" + Description + "'," + breakLine
                    + "\t DefaultValue : '" + DefaultValue + "'." + breakLine
                    + "\t PrivateValue : '" + PrivateValue + "'." + breakLine;
        }
    }
}
