﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MyMediaProject.RecommenderSystem.Core.Config
{
    /// <summary>
    /// UI to ask user to specify path to a folder
    /// </summary>
    public partial class FolderSpecificationControl : UserControl
    {
        /// <summary>
        /// Constructor (required to initialize)
        /// </summary>
        public FolderSpecificationControl()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Asks user to specify the location of the configuration folder
        /// </summary>
        /// <returns>Path to the user specified folder</returns>
        public String AskConfigurationFolderPath()
        {
            return AskConfigurationFolderPath(true);
        }

        /// <summary>
        /// Asks user to specify the location of the configuration folder
        /// </summary>
        /// <param name="isRequired">if true, user will be forced to specify the folder</param>
        /// <returns>Path to the user specified folder</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public String AskConfigurationFolderPath(bool isRequired)
        {
            this.Focus();
            this.BringToFront();
            this.Show();
            String PathToConfigDir = null;

            // for some reason there is no way to specify custom initial folder in this component.
            ConfigurationFolderBrowserDialog.RootFolder = Environment.SpecialFolder.MyComputer;

            DialogResult specifyFolderDialogResult = ConfigurationFolderBrowserDialog.ShowDialog();
            if (specifyFolderDialogResult == DialogResult.OK)
            {
                PathToConfigDir = ConfigurationFolderBrowserDialog.SelectedPath;
            }

            // if the folder is chosen, or if there is a previous value of that setting
            if (!isRequired || !String.IsNullOrEmpty(PathToConfigDir))
            {
                return PathToConfigDir;
            }

            DialogResult result = MessageBox.Show(this, "The application requires the configuration folder's location to be specified.\r\n", "Warning", MessageBoxButtons.RetryCancel, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1);
            if (result == DialogResult.Retry)
            {
                InitializeComponent();
                return AskConfigurationFolderPath(isRequired);
            }
            else if (result == DialogResult.Cancel)
            {
                throw new OperationCanceledException();
            }

            return null;
        }
    }
}
