﻿using System.Collections.Generic;
using System;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Every part of the recommender flow, that wants to receive context status messages must implement
    /// this interface. By implementing this interface, the delivery of context status items is enabled.
    /// </summary>
    public interface IContextSink
    {
        /// <summary>
        /// The id of the context sink.
        /// </summary>
        RecommenderId Id
        {
            get;
        }

        /// <summary>
        /// This function must return a list of all context status object types that
        /// the context sink wants to get notifications for.
        /// </summary>
        /// <returns>A list of context status object types.</returns>
        IEnumerable<Type> HandledContextTypes();

        /// <summary>
        /// This function will be called by the context broker, whenever a
        /// context status object changes.
        /// </summary>
        /// <param name="contextStatus">The new value of the context status object.</param>
        void ReceiveContextStatus(ContextStatus contextStatus);
    }
}
