﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EMIC.Design;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Represents a generic action between a <see cref="User"/> and <see cref="CatalogItem"/>.
    /// </summary>
    /// <remarks>
    /// This is an extensibility mechanism to allow additional information about the user to be captured by the system.
    /// </remarks>
    partial class OtherAction
    {
        private string name;
        private string property;

        /// <summary>
        /// Initializes a new instance of the <see cref="OtherAction"/> class.
        /// </summary>
        /// <param name="user">The user.</param>
        /// <param name="item">The item.</param>
        /// <param name="actionName">Name of the action.</param>
        /// <param name="actionProperty">The action property.</param>
        public OtherAction(User user, CatalogItem item, string actionName, string actionProperty) :
            base(user, item)
        {
            this.name = actionName;
            this.property = actionProperty;
        }

        internal override void PrepareForInsert(MyMediaDataContext dbContext)
        {
            OtherActionProperty actionProperty = dbContext.OtherActionProperties.SingleOrDefault<OtherActionProperty>(
                ap => ap.Name == this.name && ap.Property == this.property);
            if (null == actionProperty)
            {
                actionProperty = new OtherActionProperty(this.name, this.property);
                dbContext.OtherActionProperties.InsertOnSubmit(actionProperty);
            }
            actionProperty.UserActions.Add(this);
            base.PrepareForInsert(dbContext);
        }
        /// <summary>
        /// Lambda expression to determine if two UserAction derived classes are logically equivalent.
        /// </summary>
        /// <returns></returns>
        public override System.Linq.Expressions.Expression<Func<UserAction, bool>> EquivalenceQuery()
        {
            Contract.Precondition(null != this.name);
            return otherAction => otherAction.OtherActionProperty.Name == this.name;
        }

        /// <summary>
        /// Resolves the equivalent action.
        /// </summary>
        /// <param name="existingAction">The existing action.</param>
        public override void ResolveEquivalentAction(UserAction existingAction)
        {
            existingAction.OtherActionProperty.Property = this.property;
            base.ResolveEquivalentAction(existingAction);
        }
    }
}