﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Represents an association of text to a <see cref="CatalogItem"/> by a <see cref="User"/>.
    /// </summary>
    partial class TaggingAction
    {
        private string tag;

        /// <summary>
        /// Initializes a new instance of the <see cref="TaggingAction"/> class.
        /// </summary>
        /// <param name="user">The user.</param>
        /// <param name="item">The item.</param>
        /// <param name="tag">The tag.</param>
        public TaggingAction(User user, CatalogItem item, string tag)
            : base(user, item)
        {
            this.tag = tag;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TaggingAction"/> class.
        /// </summary>
        /// <param name="user">The user.</param>
        /// <param name="item">The item.</param>
        /// <param name="tag">The tag.</param>
        /// <param name="timestamp">The timestamp.</param>
        public TaggingAction(User user, CatalogItem item, string tag, DateTime timestamp)
            : base(user, item, timestamp)
        {
            this.tag = tag;
        }

        /// <summary>
        /// Prepares the item for insert by reconciling any external tables.
        /// </summary>
        /// <param name="dbContext">The db context.</param>
        internal override void PrepareForInsert(MyMediaDataContext dbContext)
        {
            TagValue tagValue = dbContext.TagValues.SingleOrDefault<TagValue>(
                tv => tv.Tag == this.tag);
            if (null == tagValue)
            {
                tagValue = new TagValue(this.tag);
                dbContext.TagValues.InsertOnSubmit(tagValue);
            }
            tagValue.UserActions.Add(this);
            base.PrepareForInsert(dbContext);
        }
        /// <summary>
        /// Lambda expression to determine if two UserAction derived classes are logically equivalent.
        /// </summary>
        public override System.Linq.Expressions.Expression<Func<UserAction, bool>> EquivalenceQuery()
        {
            return action => (action as TaggingAction).TagValue.Tag == this.tag;
        }

        /// <summary>
        /// Resolves the equivalent action.
        /// </summary>
        /// <param name="existingAction">The existing action.</param>
        public override void ResolveEquivalentAction(UserAction existingAction)
        {
            //No specific action required here; the tag is equivalent, so simply allow the base class to update as appropriate.
            base.ResolveEquivalentAction(existingAction);
        }
    }
}
