﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm
{
    /// <summary>
    /// Data reader abstraction for reading data for an <see cref="EntityRelationEngine"/>
    /// </summary>
    public interface IEntityRelationDataReader : IDisposable
    {
        /// <summary>
        /// Shall implement logic to open the underlying data source. 
        /// </summary>
        void Open();

        /// <summary>
        /// Shall implement logic to close the underlying data source.
        /// </summary>
        void Close();

        /// <summary>
        /// Returns the state of the connection to the underlying data source. 
        /// Data can be returned only when this method returns true.
        /// </summary>
        bool IsOpen
        {
            get;
        }

        /// <summary>
        /// Returns the number of columns exposed by the underlying data source.
        /// </summary>
        int ColumnCount
        {
            get;
        }

        /// <summary>
        /// As int as this method returns TRUE there is another row available. 
        /// It returns FALSE if there are no more rows available. In the beginning 
        /// the position of the cursor is before the first data row.
        /// </summary>
        /// <returns>Returns TRUE if the reader has been sucessfully advanced 
        /// to the next row. Returns FALSE if the reader does not have any 
        /// further rows.</returns>
        Boolean Read();

        /// <summary>
        /// Gets the int16.
        /// </summary>
        /// <param name="colIndex">Index of the col.</param>
        /// <returns></returns>
        Int16 GetInt16(int colIndex);

        /// <summary>
        /// Gets the int32.
        /// </summary>
        /// <param name="colIndex">Index of the col.</param>
        /// <returns></returns>
        Int32 GetInt32(int colIndex);

        /// <summary>
        /// Gets the int64.
        /// </summary>
        /// <param name="colIndex">Index of the col.</param>
        /// <returns></returns>
        Int64 GetInt64(int colIndex);

        /// <summary>
        /// Gets the double.
        /// </summary>
        /// <param name="colIndex">Index of the col.</param>
        /// <returns></returns>
        Double GetDouble(int colIndex);

        /// <summary>
        /// Gets the string.
        /// </summary>
        /// <param name="colIndex">Index of the col.</param>
        /// <returns></returns>
        String GetString(int colIndex);
    }
}
