﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Core.Config;
using System.Data;
using System.Data.Common;
using System.Configuration;


namespace MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm
{
    /// <summary>
    /// A data reader implementation to transform information contained in a <see cref="DbDataReader"/> to a
    /// <see cref="IEntityRelationDataReader"/>.
    /// </summary>
    public class SqlEntityRelationDataReader : IEntityRelationDataReader
    {
        private DbDataReader m_dbDataReader;
        private bool m_isOpen = true;

        /// <summary>
        /// Initializes a new instance of the <see cref="SqlEntityRelationDataReader"/> class.
        /// </summary>
        /// <param name="reader">The reader.</param>
        public SqlEntityRelationDataReader(DbDataReader reader)
        {
            this.m_dbDataReader = reader;
        }

        /// <summary>
        /// Shall implement logic to open the underlying data source.
        /// </summary>
        public void Open()
        {
        }

        /// <summary>
        /// Shall implement logic to close the underlying data source.
        /// </summary>
        public void Close()
        {
            m_dbDataReader.Close();
            m_isOpen = false;
        }

        /// <summary>
        /// Returns the state of the connection to the underlying data source.
        /// Data can be returned only when this method returns true.
        /// </summary>
        /// <value></value>
        public bool IsOpen
        {
            get { return m_isOpen; }
        }

        /// <summary>
        /// Returns the number of columns exposed by the underlying data source.
        /// </summary>
        /// <value></value>
        public int ColumnCount
        {
            get { return m_dbDataReader.FieldCount; }
        }

        /// <summary>
        /// As int as this method returns TRUE there is another row available.
        /// It returns FALSE if there are no more rows available. In the beginning
        /// the position of the cursor is before the first data row.
        /// </summary>
        /// <returns>
        /// Returns TRUE if the reader has been sucessfully advanced
        /// to the next row. Returns FALSE if the reader does not have any
        /// further rows.
        /// </returns>
        public bool Read()
        {
            return m_dbDataReader.Read();
        }

        /// <summary>
        /// Gets the int16.
        /// </summary>
        /// <param name="colIndex">Index of the col.</param>
        /// <returns></returns>
        public short GetInt16(int colIndex)
        {
            return m_dbDataReader.GetInt16(colIndex);
        }

        /// <summary>
        /// Gets the int32.
        /// </summary>
        /// <param name="colIndex">Index of the col.</param>
        /// <returns></returns>
        public int GetInt32(int colIndex)
        {
            return m_dbDataReader.GetInt32(colIndex);
        }

        /// <summary>
        /// Gets the int64.
        /// </summary>
        /// <param name="colIndex">Index of the col.</param>
        /// <returns></returns>
        public long GetInt64(int colIndex)
        {
            return m_dbDataReader.GetInt64(colIndex);
        }

        /// <summary>
        /// Gets the double.
        /// </summary>
        /// <param name="colIndex">Index of the col.</param>
        /// <returns></returns>
        public double GetDouble(int colIndex)
        {
            return m_dbDataReader.GetDouble(colIndex);
        }

        #region IDataReader Members


        /// <summary>
        /// Gets the string.
        /// </summary>
        /// <param name="colIndex">Index of the col.</param>
        /// <returns></returns>
        public string GetString(int colIndex)
        {
            throw new NotImplementedException();
        }

        #endregion

        #region IDisposable Members

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                this.m_dbDataReader.Dispose();
                this.m_isOpen = false;
            }
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        #endregion
    }
}
