﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Defines a method that generates predictions (recommendations) for all items in the query.
    /// </summary>
    public interface IPredictionAlgorithm : IPredictionAlgorithm<CatalogItem>
    {
    }
    
    /// <summary>
    /// Defines a method that generates predictions (recommendations) for all items in the query.
    /// </summary>
    /// <typeparam name="TCatalogItemType">The type of the catalog item type.</typeparam>
    public interface IPredictionAlgorithm<TCatalogItemType>
                where TCatalogItemType : CatalogItem
    {
        /// <summary>
        /// Calculates a prediction (recommendation) for each catalog item in the query.
        /// </summary>
        /// <param name="query">The query.</param>
        /// <param name="user">The user.</param>
        /// <returns>a list of predictions.</returns>
        /// <remarks>
        /// This forms the heart of the recommendation algorithm. It has to generate a prediction for each catalog item in the query.
        /// </remarks>
        IList<Prediction<TCatalogItemType>> Predict(IEnumerable<TCatalogItemType> query, User user);
    }
}
