﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.Globalization;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Defines an explanation that is part of a <see cref="Prediction&lt;TCatalogItemType&gt;"/>.
    /// </summary>
    [Serializable]
    public class Explanation
    {
        /// <summary>
        /// Gets or sets the kind of the explanation.
        /// </summary>
        /// <value>The kind (i.e. keyword, person).</value>
        public string Kind { get; set; }

        /// <summary>
        /// Gets or sets the description of the explanation.
        /// </summary>
        /// <value>The description (i.e. fishing, John Wayne).</value>
        public string Description { get; set; }

        /// <summary>
        /// Gets or sets the value/ weight of the explanation.
        /// </summary>
        /// <value>The value/ weight.</value>
        public double? Value { get; set; }

        /// <summary>
        /// Gets or sets a list of sub explanations.
        /// </summary>
        /// <value>The sub explanations.</value>
        public Collection<Explanation> SubExplanations { get; private set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="Explanation"/> class.
        /// </summary>
        public Explanation()
        {
            SubExplanations = new Collection<Explanation>();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Explanation"/> class.
        /// </summary>
        /// /// <param name="kind">The kind.</param>
        /// <param name="description">The description.</param>
        /// <param name="value">The value.</param>
        public Explanation(string kind, string description, double? value)
        {
            SubExplanations = new Collection<Explanation>();
            this.Kind = kind;
            this.Description = description;
            this.Value = value;
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            string result = "";

            if (Kind != null)
            {
                result = Kind;
            }
            if (!string.IsNullOrEmpty(Description) || Value != null)
            {
                result += ": ";
                if (!string.IsNullOrEmpty(Description))
                {
                    result += Description+ " ";
                }
                if (Value != null)
                {
                    result += string.Format(CultureInfo.InvariantCulture,"({0}%) ",(Value.Value * 100));
                }
            }                
            if (SubExplanations.Count > 0)
            {
                result += "{";
                foreach (Explanation e in SubExplanations)
                {
                    result += e.ToString()+ ",";
                }
                result += "}";
            }
            return result;
        }
    }
}
