﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Defines a method to hybridize (combine) the results of (multiple) <see cref="Engine&lt;TCatalogItemType, TUserActionType&gt;"/> to a single
    /// <see cref="Prediction&lt;TCatalogItemType&gt;"/>.
    /// </summary>
    public interface IHybridizer : IHybridizer<CatalogItem>
    {
    }

    /// <summary>
    /// Defines a method to hybridize (combine) the results of (multiple) <see cref="Engine&lt;TCatalogItemType, TUserActionType&gt;"/> to a single
    /// <see cref="Prediction&lt;TCatalogItemType&gt;"/>.
    /// </summary>
    /// <typeparam name="TCatalogItemType">The catalog item type.</typeparam>
    public interface IHybridizer<TCatalogItemType>
        where TCatalogItemType : CatalogItem
    {
        /// <summary>
        /// Hybridizes the specified engine results.
        /// </summary>
        /// <param name="engineResults">The engine results.</param>
        /// <returns>a list of a prediction per catalog item.</returns>
        IList<Prediction<TCatalogItemType>> Hybridize(ReadOnlyDictionary<TCatalogItemType, IList<EngineResult<TCatalogItemType>>> engineResults);
    }
}
