﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Defines a method to filter our catalog items during a prediction on a <see cref="Flow&lt;TCatalogItemType, TUserActionType&gt;"/>.
    /// </summary>
    /// <remarks>
    /// PostFilters are called after engines (<see cref="Engine&lt;TCatalogItemType, TUserActionType&gt;"/>in a prediction process. Filters decide
    /// whether an item is part of the predictions or not.
    /// </remarks>
    /// <typeparam name="TCatalogItemType">The catalog item type.</typeparam>
    public interface IPostFilter<TCatalogItemType>
        where TCatalogItemType : CatalogItem
    {
        /// <summary>
        /// Defines if an item is part of the predictions (<see cref="Flow&lt;TCatalogItemType, TUserActionType&gt;"/>) or not.
        /// </summary>
        /// <param name="predictions">The predictions.</param>
        /// <returns>a (reduced) list of predictions.</returns>
        IList<Prediction<TCatalogItemType>> PostFilter(ReadOnlyCollection<Prediction<TCatalogItemType>> predictions);
    }
}
