﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// This hybridizer can be used if a single engine flow is needed and turns an <see cref="Engine&lt;TCatalogItemType, TUserActionType&gt;"/> (engine result) directly
    /// into a <see cref="Prediction&lt;TCatalogItemType&gt;"/>.
    /// It is the default hybridizer for a new flow.
    /// </summary>
    public class SingleEngineHybridizer : SingleEngineHybridizer<CatalogItem>, IHybridizer
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SingleEngineHybridizer"/> class.
        /// </summary>
        public SingleEngineHybridizer() { }
    }

    /// <summary>
    /// This hybridizer can be used if a single engine flow is needed and turns an <see cref="Engine&lt;TCatalogItemType, TUserActionType&gt;"/> (engine result) directly
    /// into a <see cref="Prediction&lt;TCatalogItemType&gt;"/>.
    /// It is the default hybridizer for a new flow.
    /// </summary>
    /// <typeparam name="TCatalogItemType">The catalog item type.</typeparam>
    public class SingleEngineHybridizer<TCatalogItemType> : IHybridizer<TCatalogItemType>
        where TCatalogItemType : CatalogItem
    {
        internal SingleEngineHybridizer() { }

        #region IHybridizer<TCatalogItemType> Members

        /// <summary>
        /// Hybridizes a single engine result.
        /// Throws an exception if multiple engine results are send to it.
        /// </summary>
        /// <param name="engineResults">The engine results.</param>
        /// <returns>a list of a prediction per catalog item.</returns>
        public IList<Prediction<TCatalogItemType>> Hybridize(ReadOnlyDictionary<TCatalogItemType, IList<EngineResult<TCatalogItemType>>> engineResults)
        {
            List<Prediction<TCatalogItemType>> result = new List<Prediction<TCatalogItemType>>();

            foreach (IList<EngineResult<TCatalogItemType>> engineResult in engineResults.Values)
            {
                if (engineResult.Count > 1)
                    throw new ArgumentException("This hybridizer can only be used for single engine flows.");

                result.Add(engineResult[0].Prediction);
            }

            return result;
        }

        #endregion
    }
}
