﻿using System;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Identifies a logical store.
    /// </summary>
    [Serializable]
    public class PersistentStoreId : RecommenderId
    {
        private string friendlyName;

        /// <summary>
        /// Initializes a new instance of the <see cref="PersistentStoreId"/> class.
        /// </summary>
        /// <param name="id">The id.</param>
        public PersistentStoreId(Guid id)
            : base(id)
        {
            this.friendlyName = id.ToString();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PersistentStoreId"/> class.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="friendlyName">Name of the friendly.</param>
        public PersistentStoreId(Guid id, string friendlyName) : base(id)
        {
            this.friendlyName = friendlyName;
        }

        /// <summary>
        /// Gets the friendly name of the store.
        /// </summary>
        /// <value>The friendly name of the store.</value>
        public string FriendlyName
        {
            get { return friendlyName; }
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            return this.friendlyName;
        }
    }
}
