﻿using MyMediaProject.RecommenderSystem.Core;
using System;
using EMIC.Design;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Provides data for the <see cref="ProgressEvent"/>.
    /// </summary>
    [Serializable]
    public class ProgressEventArgs: RecommenderEventArgs
    {
        private int percentage;
        private bool finished;

        /// <summary>
        /// Gets or sets the percentage.
        /// </summary>
        /// <value>The percentage.</value>
        public int Percentage
        {
            get
            {
                return this.percentage;
            }
            private set
            {
                Contract.Assert(percentage >= 0 && percentage <= 100, "Percantage must be in the range [0;100]");
                this.percentage = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="ProgressEventArgs"/> is finished.
        /// </summary>
        /// <value><c>true</c> if finished; otherwise, <c>false</c>.</value>
        public bool Finished
        {
            get { return finished; }
            set { finished = value; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ProgressEventArgs"/> class.
        /// </summary>
        /// <param name="percentage">The percentage.</param>
        /// <param name="finished">if set to <c>true</c> [finished].</param>
        public ProgressEventArgs(int percentage, bool finished)
        {
            this.Percentage = percentage;
            this.finished = finished;
        }
    }
}