using System;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Identifies the store change.
    /// </summary>
    public enum StoreChangeType
    {
        /// <summary>
        /// An item has been added.
        /// </summary>
        Added,
        /// <summary>
        /// An item has been modified.
        /// </summary>
        Changed,
        /// <summary>
        /// An item has been removed.
        /// </summary>
        Removed 
    } ;

    /// <summary>
    /// Identifies the change occuring in a logical store.
    /// </summary>
    [Serializable]
    public class StoreChange : IStoreChange
    {
        /// <summary>
        /// The change type.
        /// </summary>

        private int changeNumber;
        private StoreChangeType changeType;
        private RecommenderId itemId;

        /// <summary>
        /// Gets the change number.
        /// </summary>
        /// <value>The change number.</value>
        public int ChangeNumber
        {
            get { return changeNumber; }
            set { changeNumber = value; }
        }

        /// <summary>
        /// Gets the type of the change.
        /// </summary>
        /// <value>The type of the change.</value>
        public StoreChangeType ChangeType
        {
            get { return changeType; }
            set { changeType = value; }
        }

        /// <summary>
        /// Gets the item id.
        /// </summary>
        /// <value>The item id.</value>
        public RecommenderId ChangedItemId
        {
            get { return itemId; }
            set { itemId = value; }
        }

        internal StoreChange(int changeNumber, StoreChangeType changeType, RecommenderId itemId)
        {
            this.changeNumber = changeNumber;
            this.changeType = changeType;
            this.itemId = itemId;
        }
    }
}
