﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Defines methods for retrieving and storing catalog items.
    /// </summary>
    public interface ICatalogItemProvider : ICatalogItemProvider<CatalogItem>
    {
    }

    /// <summary>
    /// Defines methods for retrieving and storing catalog items.
    /// </summary>
    /// <typeparam name="TCatalogItemType">The catalog item type.</typeparam>
    public interface ICatalogItemProvider<TCatalogItemType>
    {
        /// <summary>
        /// Gets the catalog changes.
        /// </summary>
        /// <param name="startChangeNumber">The start change number.</param>
        /// <param name="endChangeNumber">The end change number.</param>
        /// <returns>a list with all changes.</returns>
        IList<IDataStoreChange> GetCatalogChanges(int startChangeNumber, int endChangeNumber);

        /// <summary>
        /// Gets the catalog item.
        /// </summary>
        /// <param name="catalogItemId">The catalog item id.</param>
        /// <param name="catalogItem">The catalog item.</param>
        void GetCatalogItem(int catalogItemId, out TCatalogItemType catalogItem);

        /// <summary>
        /// Determines whether the provider contains a catalog item with the specified catalog item id.
        /// </summary>
        /// <param name="catalogItemId">The catalog item id.</param>
        /// <returns>
        /// 	<c>true</c> if the provider contains a catalog item with the specified catalog item id]; otherwise, <c>false</c>.
        /// </returns>
        bool ContainsCatalogItem(int catalogItemId);

        /// <summary>
        /// Retrieves catalog items.
        /// </summary>
        /// <returns>The catalog items</returns>
        IEnumerable<TCatalogItemType> RetrieveCatalogItems();

        /// <summary>
        /// Gets the latest (most recent, or "tip") change number.
        /// </summary>
        /// <value>The latest change number.</value>
        int ChangeNumber
        {
            get;
        }
    }
}
