﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MyMediaProject.RecommenderSystem.Core
{
    /// <summary>
    /// Defines methods for retrieving and storing feedback.
    /// </summary>
    /// <typeparam name="TCatalogItemType">The catalog item type.</typeparam>
    /// <typeparam name="TUserActionType">The feedback item type.</typeparam>
    internal interface IUserActionProvider<TCatalogItemType, TUserActionType>
        where TCatalogItemType : CatalogItem
        where TUserActionType : UserAction
    {
        /// <summary>
        /// Gets the feedback changes.
        /// </summary>
        /// <param name="startChangeNumber">The start change number.</param>
        /// <param name="endChangeNumber">The end change number.</param>
        /// <returns>a list with all changes.</returns>
        IList<IDataStoreChange> GetFeedbackChanges(int startChangeNumber, int endChangeNumber);

        /// <summary>
        /// Retrieves the feedback item.
        /// </summary>
        /// <param name="feedbackItemId">The feedback item id.</param>
        /// <param name="feedbackItem">The feedback item.</param>
        /// <param name="catalogItem">The catalog item.</param>
        void RetrieveUserAction(int feedbackItemId, out TUserActionType feedbackItem, out TCatalogItemType catalogItem);


        /// <summary>
        /// Retrieves previous (in time) user specific feedback.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="maxHistoryLength">The maximum amount of feedback items to return</param>
        /// <returns>The feedback</returns>
        IEnumerable<TUserActionType> RetrieveLastFeedback(User userId, int maxHistoryLength);


        /// <summary>
        /// Gets the latest (most recent, or "tip") change number.
        /// </summary>
        /// <value>The latest change number.</value>
        int ChangeNumber
        {
            get;
        }
    }
}
