﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MyMediaProject.RecommenderSystem.Core;
using System.Diagnostics;

namespace RecommenderCoreTest
{
    [TestClass]
    public class CoreBenchmark
    {
        private ProgramId programId = new ProgramId(new Guid("1660237F-64FA-43e8-8230-F8C85FC66E8B"), "CoreBenchmark");
        private BenchmarkItemFactory factory;
        private Recommender<UserAction> recommender;
        private Flow<CatalogItem, UserAction> flow;
        private User userId = new User(){UserName = "CoreBenchmarkUser"};

        private void RunBenchmark(int number)
        {
            DateTime startTime;
            IQueryable<CatalogItem> items;

            startTime = DateTime.Now;
            Trace.TraceInformation(string.Format("Creating {0} items...", new object[] { number }));
            // Create the items
            factory.CreateItems(number);
            Trace.TraceInformation(string.Format("{0} - Finished creating {1} items, {2} items per second.", new object[] { DateTime.Now.Subtract(startTime).ToString(), number, (number / DateTime.Now.Subtract(startTime).TotalMilliseconds) * 1000 }));

            items = recommender.CatalogItemStore;
            startTime = DateTime.Now;
            Trace.TraceInformation(string.Format("Now get recommendations for all items...", new object[] { }));
            // Now get predictions for all of the items     
            recommender.Predict(items, flow.Id, userId);
            Trace.TraceInformation(string.Format("{0} - Finished getting recommendations, {1} items per second.", new object[] { DateTime.Now.Subtract(startTime).ToString(), (number / DateTime.Now.Subtract(startTime).TotalMilliseconds) * 1000 }));
        }

        //[TestMethod]
        [Timeout(10000000)]
        [Ignore]
        public void RecommenderCoreBenchmark()
        {
            int smallNumber = 100;
            recommender = new Recommender<UserAction>(programId);
            flow = new Flow<CatalogItem,UserAction>(new FlowId(Guid.NewGuid()));
            FlowTestAveragingHybridizer<CatalogItem> hybridizer = new FlowTestAveragingHybridizer<CatalogItem>();
            FlowTestFixedRatingEngine<CatalogItem, UserAction> engine = new FlowTestFixedRatingEngine<CatalogItem, UserAction>();
            
            factory = new BenchmarkItemFactory();

            flow.Hybridizer = hybridizer;
            flow.AddEngine(engine);

            recommender.Initialize();

            try
            {
                recommender.AddUsers(new User[] { userId });
            }
            catch
            { 
            }

            recommender.RegisterCatalogFactory(factory);

            recommender.RegisterFlow(flow);

            factory.DeleteAllItems();
            // First run with a small number of items
            RunBenchmark(smallNumber);

            factory.DeleteAllItems();
            // First run with a medium number of items
            RunBenchmark(smallNumber * 10);

            factory.DeleteAllItems();
            // Now run with a large number of items
            RunBenchmark(smallNumber * 100);

            //factory.DeleteAllItems();
            //// Now run with a huge number of items
            //RunBenchmark(smallNumber * 1000);
        }
    }  
}
