﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Globalization;
using System.Threading;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Core.Databases;

namespace RecommenderCoreTest.Databases.CatalogItemStoreTest
{
    /// <summary>
    /// Summary description for CatalogItemTest
    /// </summary>
    [TestClass]
    public class ConcurrencyTest
    {
        ProgramId programId = new ProgramId(new Guid("{79029808-8D5D-4c7d-9201-DA005E150C77}"),
            "Test store for the ConcurrencyTest class.");
        Recommender recommender;
        string userNameFormatter = "User{0}";
        int threadIterations = 100;
        bool endTests = false;

        public ConcurrencyTest()
        {
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        [DeploymentItem("Config\\MyMedia.mmconf.xml", "Config")]
        public void UserConcurrencyTest()
        {
            List<User> users = new List<User>();
            this.recommender = new Recommender(this.programId);
            this.recommender.Initialize();

            this.endTests = false;

            recommender.ResetAllSettings();

            for (int i = 0; i < 1000; i++)
            {
                User user = new User();
                user.UserName = string.Format(this.userNameFormatter, i);
                users.Add(user);
            }
            this.recommender.AddUsers(users.ToArray());

            int threadCount = 2;
            List<Thread> activeThreads = new List<Thread>();

            for (int i = 0; i < threadCount; i++)
            {
                Thread thread = new Thread(new ThreadStart(this.UserConcurrencyTestThread));
                thread.Name = string.Format("ConcurrencyTest Thread {0}", i);
                thread.Start();
                activeThreads.Add(thread);
            }
            Thread.Sleep(60000);
            this.endTests = true;
            foreach(Thread thread in activeThreads)
            {
                thread.Join(1000);
            }
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        private void UserConcurrencyTestThread()
        {
            for (int i = 0; i < this.threadIterations; i++)
            {
                int userCount = 0;
                using (IReleasableQueryable<User> users = this.recommender.Users)
                {
                    userCount = users.Count();
                }
                for (int j = 0; j < userCount; j++)
                {
                    if (this.endTests)
                    {
                        break;
                    }
                    using (IReleasableQueryable<User> users = this.recommender.Users)
                    {
                        string userName = string.Format(this.userNameFormatter, j);
                        User user = users.Single(u => u.UserName == userName);
                    }
                }
                if (this.endTests)
                {
                    break;
                }
            }
        }
    }
}
