﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Globalization;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Core.Databases;

namespace RecommenderCoreTest.Databases
{
    /// <summary>
    /// Summary description for UserTest
    /// </summary>
    [TestClass]
    public class UserTest
    {
        ProgramId userTestProgramId = new ProgramId(new Guid("{8B8D95BD-BEFA-48a8-99F3-DD3D4E556723}"),
            "Test store for the UserTest class.");
        
        public UserTest()
        {
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        [DeploymentItem("Config\\MyMedia.mmconf.xml", "Config")]
        public void UserDataStoreTests()
        {
            using (Recommender recommender = new Recommender(userTestProgramId))
            {
                recommender.Initialize();
                recommender.ResetAllSettings();
                List<User> users = new List<User>();
                for (int i = 0; i < 1000; i++)
                {
                    User user = new User();
                    user.UserName = i.ToString();
                    users.Add(user);
                }
                recommender.AddUsers(users.ToArray());
                int userObjectsCreatedBefore = 0;
                int userObjectsCreatedAfter = 0;
                int itemCount = 0;
                IQueryable<User> query;

                string userName = "BillyBob";
                string propName = "Age";
                string prop2Name = "Gender";
                string propValue = "30";
                string propValueUpdate = "60";
                string prop2Value = "M";

                userObjectsCreatedBefore = RecommenderStatistics.UsersCreated;
                using (IReleasableQueryable<User> userQuery = recommender.Users)
                {
                    List<User> queryList;
                    itemCount = userQuery.Count();
                    userObjectsCreatedAfter = RecommenderStatistics.UsersCreated;
                    Assert.AreEqual<int>(userObjectsCreatedBefore, userObjectsCreatedAfter, "No new user objects created during get count query for all items.");
                    Assert.AreEqual<int>(1000, itemCount, "Verify correct number of users");

                    int takeCount = 20;
                    userObjectsCreatedBefore = RecommenderStatistics.UsersCreated;
                    itemCount = userQuery.Take(takeCount).Count();
                    userObjectsCreatedAfter = RecommenderStatistics.UsersCreated;
                    Assert.AreEqual<int>(userObjectsCreatedBefore, userObjectsCreatedAfter, "No new user objects created during get count query for subset of items.");
                    Assert.AreEqual<int>(takeCount, itemCount, "Verify correct number of users");

                    userObjectsCreatedBefore = RecommenderStatistics.UsersCreated;
                    query = userQuery.Take(takeCount);
                    queryList = query.ToList(); //Force execution of query.
                    userObjectsCreatedAfter = RecommenderStatistics.UsersCreated;
                    Assert.AreEqual<int>(takeCount, userObjectsCreatedAfter - userObjectsCreatedBefore, "Number of items created is same as those queried, using .Take().");
                    Assert.AreEqual<int>(takeCount, queryList.Count(), "Verify correct number of users in query.");

                    userObjectsCreatedBefore = RecommenderStatistics.UsersCreated;
                    takeCount = 50;
                    query = userQuery.Where(u => Convert.ToInt32(u.UserName) < takeCount);
                    queryList = query.ToList(); //Force execution of query.
                    userObjectsCreatedAfter = RecommenderStatistics.UsersCreated;
                    Assert.AreEqual<int>(takeCount, userObjectsCreatedAfter - userObjectsCreatedBefore, "Number of items created is same as those queried, using .Take().");
                    Assert.AreEqual<int>(takeCount, queryList.Count(), "Verify correct number of users in query.");

                    userObjectsCreatedBefore = RecommenderStatistics.UsersCreated;
                    User newUser = new User();
                    newUser.UserName = userName;
                    newUser.AddProperty(recommender, propName, propValue);
                    newUser.AddProperty(recommender, prop2Name, prop2Value);
                    recommender.AddUsers(new User[] { newUser });
                    Assert.AreEqual<string>(propValue, newUser.UserProperties.Single(itemProp => itemProp.Property.Name == propName).Property.Value, "Property value not added correctly.");
                    Assert.AreEqual<string>(prop2Value, newUser.UserProperties.Single(itemProp => itemProp.Property.Name == prop2Name).Property.Value, "Property value not added correctly.");

                    using (UserUpdate userUpdate = recommender.GetUpdateableUsers())
                    {
                        User updatedUser = userUpdate.First();
                        updatedUser.UserName = updatedUser.UserName + "Updated";
                        updatedUser.AddProperty(recommender, propName, propValue);
                        userUpdate.SubmitUserChanges();
                        Assert.IsTrue(updatedUser.UserProperties.Any(itemProp => itemProp.Property.Name == propName), "Property missing.");
                        Assert.AreEqual<string>(propValue, updatedUser.UserProperties.Single(itemProp => itemProp.Property.Name == propName).Property.Value, "Property value not added correctly");

                        updatedUser.AddProperty(recommender, prop2Name, prop2Value);
                        userUpdate.SubmitUserChanges();
                        Assert.IsTrue(updatedUser.UserProperties.Any(itemProp => itemProp.Property.Name == propName), "Property missing.");
                        Assert.AreEqual<string>(propValue, updatedUser.UserProperties.Single(itemProp => itemProp.Property.Name == propName).Property.Value, "Property value not added correctly");
                        Assert.IsTrue(updatedUser.UserProperties.Any(itemProp => itemProp.Property.Name == prop2Name), "Property missing.");
                        Assert.AreEqual<string>(prop2Value, updatedUser.UserProperties.Single(itemProp => itemProp.Property.Name == prop2Name).Property.Value, "Property value not added correctly");

                        updatedUser.UpdateProperty(recommender, propName, propValue, propValueUpdate);
                        userUpdate.SubmitUserChanges();
                        Assert.IsTrue(updatedUser.UserProperties.Any(itemProp => itemProp.Property.Name == propName), "Property missing.");
                        Assert.AreEqual<string>(propValueUpdate, updatedUser.UserProperties.Single(itemProp => itemProp.Property.Name == propName).Property.Value, "Property value not added correctly");
                        Assert.IsTrue(updatedUser.UserProperties.Any(itemProp => itemProp.Property.Name == prop2Name), "Property missing.");
                        Assert.AreEqual<string>(prop2Value, updatedUser.UserProperties.Single(itemProp => itemProp.Property.Name == prop2Name).Property.Value, "Property value not added correctly");
                    }
                }
                using (IReleasableQueryable<User> newUserQuery = recommender.Users)
                {
                    userObjectsCreatedBefore = RecommenderStatistics.UsersCreated;
                    query = newUserQuery.Where(u => u.UserProperties.Any(p => p.Property.Name == propName));

                    Assert.AreEqual<int>(2, query.Count(), "Two users with properties not found.");

                    User newUser = query.FirstOrDefault();
                    userObjectsCreatedAfter = RecommenderStatistics.UsersCreated;
                    Assert.AreNotEqual<User>(null, newUser, "User not found.");
                    Assert.AreEqual<int>(1, userObjectsCreatedAfter - userObjectsCreatedBefore, "Number of items created is same as those queried.");
                    Assert.IsTrue(newUser.UserProperties.Any(p => p.Property.Name == propName), "User does not have the required property.");
                }
                {
                    //User user = recommender.Users.Single(u => u.UserName  == userName);
                    //string value = user.UserProperties.Single(up => up.Property.Name == propName).Property.Value;
                    //user.UpdateProperty(recommender, propName, value, value + "New");
                    //value = user.UserProperties.Single(up => up.Property.Name == prop2Name).Property.Value;
                    //user.UpdateProperty(recommender, prop2Name, value, value + "New");
                }
            }
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }
    }
}
