﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MyMediaProject.RecommenderSystem.Core;

namespace RecommenderCoreTest
{
    public class FlowTestAveragingHybridizer<TCatalogItemType> : IHybridizer<TCatalogItemType>
        where TCatalogItemType : CatalogItem
    {
        private Prediction<TCatalogItemType> Hybridize(TCatalogItemType catalogItem, IList<EngineResult<TCatalogItemType>> engineResult)
        {
            double sum = 0;
            Explanation explanation = new Explanation();

            foreach (EngineResult<TCatalogItemType> er in engineResult)
            {
                if (er.Prediction != null)
                {
                    sum += er.Prediction.Rating;
                    if (er.Prediction.Explanation != null)
                    {

                        explanation.SubExplanations.Add(er.Prediction.Explanation);
                    }
                }
            }

            Prediction<TCatalogItemType> result = new Prediction<TCatalogItemType>(catalogItem);
            result.Rating = sum / engineResult.Count;
            result.Explanation = explanation;

            return result;
        }

        #region IHybridizer<TCatalogItemType> Members

        public IList<Prediction<TCatalogItemType>> Hybridize(ReadOnlyDictionary<TCatalogItemType, IList<EngineResult<TCatalogItemType>>> engineResults)
        {
            List<Prediction<TCatalogItemType>> result = new List<Prediction<TCatalogItemType>>(engineResults.Count);

            foreach (TCatalogItemType ci in engineResults.Keys)
                result.Add(this.Hybridize(ci, engineResults[ci]));

            return result;
        }

        #endregion
    }
}
