﻿using System;
using MyMediaProject.RecommenderSystem.Core;
using EMIC.Design;
using Microsoft.VisualStudio.TestTools.UnitTesting;
namespace RecommenderCoreTest
{
    
    
    /// <summary>
    ///This is a test class for RecommenderTest and is intended
    ///to contain all RecommenderTest Unit Tests
    ///</summary>
    [TestClass()]
    public class RecommenderTest
    {

        static private Guid InitializeTestGuid = new Guid("{5938DC26-6FAC-4872-B783-5670EEDACB44}");
        static private string IntializeTestFriendlyName = "InitializeTestFriendlyName";
        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        [ClassInitialize]
        static public void ClassInitialize(TestContext context)
        {
            ProgramId id = new ProgramId(InitializeTestGuid, IntializeTestFriendlyName);
            Recommender<UserAction> recommender = new Recommender<UserAction>(id);
            recommender.DeleteAllStores();
            recommender.Dispose();
        }

        [ClassCleanup]
        static public void ClassCleanup()
        {
            RecommenderTest.ClassInitialize(null);
        }

        [TestMethod()]
        [DeploymentItem("Config\\MyMedia.mmconf.xml", "Config")]
        public void RecommenderTestInitialize()
        {
            //Create instance 1 of the recommender.
            ProgramId id = new ProgramId(InitializeTestGuid, IntializeTestFriendlyName);
            using (Recommender<UserAction> recommender = new Recommender<UserAction>(id))
            {
                recommender.Initialize();
            }

            //Create instance 2 of the recommender
            using(Recommender<UserAction> recommender = new Recommender<UserAction>(id))
            {
                recommender.Initialize();
                Assert.IsTrue(true, "Initialization completes successfully, since dispose was called on the prior instance of the recommender.");
            }
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        [TestMethod()]
        [DeploymentItem("Config\\MyMedia.mmconf.xml", "Config")]
        [ExpectedException(typeof(PreconditionViolationException), "Two instances of the same Recommender ProgramId can not exist in the same process without disposing the prior instance.")]
        public void RecommenderTestInitializeThrow()
        {
            //Create instance 1 of the recommender.
            ProgramId id = new ProgramId(InitializeTestGuid, IntializeTestFriendlyName);
            using (Recommender<UserAction> recommender = new Recommender<UserAction>(id))
            {
                recommender.Initialize();

                //Create instance 2 of the recommender - Initialize should throw, as dispose was not called on the prior instance.
                using (Recommender<UserAction> recommender2 = new Recommender<UserAction>(id))
                {
                    recommender2.Initialize();
                }
            }
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }
    }
}
