﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Framework;

namespace RecommenderFrameworkTest.Metrics
{
    public class AscescendingOrderEngine : Engine<CatalogItem, UserAction>
    {
        static private EngineId engineId = new EngineId(new Guid("{3E4D00BB-363F-4035-B564-75462A68417F}"));
        static private string description = "A simple test engine which returns items in numerical descending order, with a prediction value of rank/query.Count().";
        public AscescendingOrderEngine()
            : base(engineId, description)
        {
        }

        public override IList<Prediction<CatalogItem>> Predict(IEnumerable<CatalogItem> query, User user)
        {
            IEnumerable<RatingAction> feedbacks =
                base.RetrieveLastFeedback(user, int.MaxValue).Where(feedback => feedback is RatingAction).Cast<RatingAction>();
            IEnumerable<CatalogItem> ratedItems = from fc in feedbacks
                                                   select fc.CatalogItem;
            IEnumerable<CatalogItem> unRatedItems = query.Except<CatalogItem>(ratedItems);
            double scalingFactor = (double) query.Count<CatalogItem>();
            IEnumerable<Prediction<CatalogItem>> predictions = from unRatedItem in unRatedItems
                                                        select new Prediction<CatalogItem>(
                                                                    unRatedItem, 
                                                                    -Convert.ToDouble(unRatedItem.ItemProperties.First(prop => prop.Property.Name == "Rank").Property.Value)/scalingFactor);
            return predictions.ToList<Prediction<CatalogItem>>();
        }

        protected override void HandleNewCatalogItems(System.Collections.ObjectModel.ReadOnlyCollection<CatalogItem> catalogItems)
        {
            //Do nothing.
        }

        protected override void HandleNewFeedback(System.Collections.ObjectModel.ReadOnlyCollection<UserAction> feedback)
        {
            //Do nothing.
        }
    }
}
