﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using System.Data.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Core.Config;
using MyMediaProject.RecommenderSystem.Framework;
using MyMediaProject.RecommenderSystem.Framework.Metrics;

namespace RecommenderFrameworkTest.Metrics
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class MetricsTests
    {
        static private ProgramId metricTestProgramId = new ProgramId(new Guid("{35FCCF35-05A5-42ab-8C41-9AB2A458D56E}"), "MetricTest");
        static private FlowId metricTestFlowId = new FlowId(new Guid("{3D04623E-2922-40e4-8D70-975537F5DE05}"));

        public MetricsTests()
        {}

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [ClassInitialize]
        static public void ClassInitialize(TestContext testContext)
        {
            ConfigManager.Initialize(true);
            MetricsTests.ClassCleanup();
        }

        [ClassCleanup]
        static public void ClassCleanup()
        {
            using(Recommender recommender = GetRecommender(metricTestFlowId, 0))
            using (RecommenderMetrics<RatingAction, RatingAction> metric = GetRecommenderMetric(recommender,
                new RandomAllButNTestSet<RatingAction>(recommender, 1),
                new MetricBase<RatingAction>[] { new RootMeanSquaredErrorMetric<RatingAction>() }))
            {
                recommender.DeleteAllStores();
            }
        }

        [TestCleanup]
        public void MetricTestCleanup()
        {
            using(Recommender recommender = GetRecommender(metricTestFlowId, 0))
            using (RecommenderMetrics<RatingAction, RatingAction> metric = GetRecommenderMetric(recommender,
                new RandomAllButNTestSet<RatingAction>(recommender, 1),
                new MetricBase<RatingAction>[] { new RootMeanSquaredErrorMetric<RatingAction>() }))
            {
                recommender.ResetAllSettings();
                metric.RemoveAllTestFeedback();
            }
        }

        [TestMethod]
        [DeploymentItem("Config\\MyMedia.mmconf.xml", "Config")]
        public void MetricTest_PerfectRecommendations_MostRecentAllButOneTestSet_ExpectedUtility()
        {
            double expected = 100.0;
            uint[] itemsToRate = new uint[] { 0, 1, 2, 3, 4 };

            using(Recommender recommender = MetricsTests.GetRecommender(metricTestFlowId, 20))
            using (RecommenderMetrics<RatingAction, RatingAction> metric = GetRecommenderMetric(recommender,
                new MostRecentAllButNTestSet<RatingAction>(recommender, 1),
                new MetricBase<RatingAction>[] { new ExpectedUtilityMetric<RatingAction>(RatingAction.MinValue) }))
            {
                AddUserRatings(recommender, itemsToRate);
                ReadOnlyDictionary<RecommenderId, double> metricResults = metric.ProduceMetrics(metricTestFlowId);

                Assert.AreEqual<string>(metricResults.Values.First<double>().ToString("E3"), expected.ToString("E3"));
            }
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        [TestMethod]
        [DeploymentItem("Config\\MyMedia.mmconf.xml", "Config")]
        public void MetricTest_RecommendationsWithError_MostRecentAllButOneTestSet_ExpectedUtility()
        {
            double expected = 87.06;
            uint[] itemsToRate = new uint[] { 1, 2, 3, 4, 5 };

            using(Recommender recommender = MetricsTests.GetRecommender(metricTestFlowId, 20))
            using (RecommenderMetrics<RatingAction, RatingAction> metric = GetRecommenderMetric(recommender,
                new MostRecentAllButNTestSet<RatingAction>(recommender, 1),
                new MetricBase<RatingAction>[] { new ExpectedUtilityMetric<RatingAction>(RatingAction.MinValue) }))
            {
                AddUserRatings(recommender, itemsToRate);
                ReadOnlyDictionary<RecommenderId, double> metricResults = metric.ProduceMetrics(metricTestFlowId);

                Assert.AreEqual<string>(metricResults.Values.First<double>().ToString("E3"), expected.ToString("E3"));
            }
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        [TestMethod]
        [DeploymentItem("Config\\MyMedia.mmconf.xml", "Config")]
        public void MetricTest_MultipleUsers_MostRecentAllButOneTestSet_ExpectedUtility()
        {
            double expected = 93.53;
            uint[] itemsToRateUser0 = new uint[] { 0, 1, 2, 3, 4 };
            uint[] itemsToRateUser1 = new uint[] { 1, 2, 3, 4, 5 };

            using(Recommender recommender = MetricsTests.GetRecommender(metricTestFlowId, 20))
            using (RecommenderMetrics<RatingAction, RatingAction> metric = GetRecommenderMetric(recommender,
                new MostRecentAllButNTestSet<RatingAction>(recommender, 1),
                new MetricBase<RatingAction>[] { new ExpectedUtilityMetric<RatingAction>(RatingAction.MinValue) }))
            {

                AddUserRatings(recommender, itemsToRateUser0);
                AddUserRatings(recommender, itemsToRateUser1);

                ReadOnlyDictionary<RecommenderId, double> metricResults = metric.ProduceMetrics(metricTestFlowId);

                Assert.AreEqual<string>(metricResults.Values.First<double>().ToString("E3"), expected.ToString("E3"));
            }
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        [TestMethod]
        [DeploymentItem("Config\\MyMedia.mmconf.xml", "Config")]
        public void MetricTest_PerfectRecommendations_RandomAllButOneTestSet_RMSE()
        {
            uint[] itemsToRate = new uint[] { 0, 1, 2, 3, 4 };
            using(Recommender recommender = MetricsTests.GetRecommender(metricTestFlowId, 20))
            using (RecommenderMetrics<RatingAction, RatingAction> metric = GetRecommenderMetric(recommender,
                new RandomAllButNTestSet<RatingAction>(recommender, 1),
                new MetricBase<RatingAction>[] { new RootMeanSquaredErrorMetric<RatingAction>() }))
            {
                AddUserRatings(recommender, itemsToRate);
                ReadOnlyDictionary<RecommenderId, double> metricResults = metric.ProduceMetrics(metricTestFlowId);

                Assert.AreEqual<double>(metricResults.Values.First<double>(), 0.0);
            }
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        [TestMethod]
        [DeploymentItem("Config\\MyMedia.mmconf.xml", "Config")]
        public void MetricTest_PerfectRecommendations_MostRecentAllButOneTestSet_RMSE()
        {
            uint[] itemsToRate = new uint[] { 0, 1, 2, 3, 4 };

            using(Recommender recommender = MetricsTests.GetRecommender(metricTestFlowId, 20))
            using (RecommenderMetrics<RatingAction, RatingAction> metric = GetRecommenderMetric(recommender,
                new MostRecentAllButNTestSet<RatingAction>(recommender, 1),
                new MetricBase<RatingAction>[] { new RootMeanSquaredErrorMetric<RatingAction>() }))
            {
                AddUserRatings(recommender, itemsToRate);
                ReadOnlyDictionary<RecommenderId, double> metricResults = metric.ProduceMetrics(metricTestFlowId);

                Assert.AreEqual<double>(metricResults.Values.First<double>(), 0.0);
            }
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        [TestMethod]
        [DeploymentItem("Config\\MyMedia.mmconf.xml", "Config")]
        public void MetricTest_RecommendationsWithError_MostRecentAllButOneTestSet_RMSE()
        {
            double ratingError = 0.25;
            uint[] itemsToRate = new uint[] { 0, 1, 2, 3, 4 };

            using(Recommender recommender = MetricsTests.GetRecommender(metricTestFlowId, 20))
            using (RecommenderMetrics<RatingAction, RatingAction> metric = GetRecommenderMetric(recommender,
                new MostRecentAllButNTestSet<RatingAction>(recommender, 1),
                new MetricBase<RatingAction>[] { new RootMeanSquaredErrorMetric<RatingAction>() }))
            {
                AddUserRatings(recommender, itemsToRate, ratingError);
                ReadOnlyDictionary<RecommenderId, double> metricResults = metric.ProduceMetrics(metricTestFlowId);

                Assert.AreEqual<double>(metricResults.Values.First<double>(), ratingError);
            }
            //Assert.AreEqual<int>(0, RecommenderStatistics.ActiveDataContexts, "There must be zero active data contexts in the system on close.");
        }

        static private RecommenderMetrics<RatingAction, RatingAction> GetRecommenderMetric(
            Recommender recommender, 
            TestSetQualifier<RatingAction> qualifier,
            MetricBase<RatingAction>[] metrics)
        {
            RecommenderMetrics<RatingAction, RatingAction> metric = new RecommenderMetrics<RatingAction, RatingAction>(
                metricTestProgramId,
                recommender,
                qualifier,
                metrics);
            return metric;
        }

        private void AddUserRatings(Recommender recommender, uint[] itemsToRate)
        {
            User userId = new User() { UserName = Guid.NewGuid().ToString() };
            recommender.AddUsers(new User[] { userId });
            RateItems(itemsToRate, recommender, userId, 0);
        }

        private void AddUserRatings(Recommender recommender, uint[] itemsToRate, double ratingDelta)
        {
            User userId = new User() { UserName = Guid.NewGuid().ToString() };
            recommender.AddUsers(new User[] { userId });
            RateItems(itemsToRate, recommender, userId, ratingDelta);
        }

        private static void RateItems(uint[] itemsToRate, Recommender recommender, User userId, double ratingDelta)
        {
            List<RatingAction> feedbacks = new List<RatingAction>();
            double scalingFactor = recommender.CatalogItemStore.Count();
            
            long startTimeTicks = DateTime.Now.Ticks;  //Need to ensure the ratings aren't all added during the same time tick.
            int count = itemsToRate.Count();

            foreach (uint itemRank in itemsToRate)
            {
                CatalogItem item = recommender.CatalogItemStore.Single(c => c.ItemProperties.Any(prop => prop.Property.Value == itemRank.ToString()));
                double ratingValue = (-Convert.ToDouble(item.ItemProperties.First(prop => prop.Property.Name == "Rank").Property.Value) / scalingFactor) + ratingDelta;
                RatingAction rating = new RatingAction(userId, item, ratingValue, new DateTime(startTimeTicks++));
                feedbacks.Add(rating);
            }
            recommender.ProcessFeedback(feedbacks.Cast<UserAction>());
        }

        private static Recommender GetRecommender(FlowId flowId, uint itemCount)
        {
            NumberedItemFactory itemFactory = new NumberedItemFactory();

            Flow<CatalogItem, UserAction> flow = new Flow<CatalogItem, UserAction>(flowId);
            flow.AddEngine(new AscescendingOrderEngine());
            flow.Hybridizer = new SingleEngineHybridizer<CatalogItem>();
            Recommender recommender = new Recommender(metricTestProgramId);
            recommender.Initialize();
            recommender.RegisterFlow(flow);
            recommender.RegisterCatalogFactory(itemFactory);
            itemFactory.AddItems(itemCount);
            return recommender;
        }
    }
}
