﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using EMIC.Design;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Framework;

namespace MyMediaProject.RecommenderSystem.Framework.Metrics
{
    /// <summary>
    /// Base class for any test set qualifier which partitions the user actions into 
    /// a test set of size N.
    /// </summary>
    /// <typeparam name="TUserActionType">The type of the user action type.</typeparam>
    public abstract class AllButNTestSetBase<TUserActionType> : TestSetQualifier<TUserActionType>
        where TUserActionType : UserAction
    {
        /// <summary>
        /// Gets or sets the test set size.
        /// </summary>
        /// <value>The test set size N.</value>
        public int TestSetSizeN { get; set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="AllButNTestSetBase&lt;TUserActionType&gt;"/> class.
        /// </summary>
        /// <param name="recommender">The recommender.</param>
        /// <param name="testSetSizeN">The test set size N.</param>
        protected AllButNTestSetBase(Recommender recommender, int testSetSizeN)
            : base(recommender)
        {
            Contract.Precondition(testSetSizeN > 0, "Test set must be greater than zero.");
            this.TestSetSizeN = testSetSizeN;
        }

        /// <summary>
        /// Extracts the test set.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="userFeedbacks">The user feedbacks.</param>
        /// <param name="testSet">The test set.</param>
        public override void ExtractTestSet(
            User userId,
            IQueryable<TUserActionType> userFeedbacks,
            Collection<TUserActionType> testSet)
        {
            if (this.TestSetSizeN < userFeedbacks.Count()) //Implies at least one item for training.
            {
                this.GetTrainingSetFromUserFeedback(
                    new Collection<TUserActionType>(userFeedbacks.ToList()),
                    testSet,
                    this.TestSetSizeN);
            }
        }

        /// <summary>
        /// Gets the training set from user feedback.
        /// </summary>
        /// <param name="feedbackItems">The feedback items.</param>
        /// <param name="toRemove">To remove.</param>
        /// <param name="count">The count.</param>
        protected abstract void GetTrainingSetFromUserFeedback(Collection<TUserActionType> feedbackItems, Collection<TUserActionType> toRemove, int count);
    }
}
