﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using EMIC.Design;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Framework;

namespace MyMediaProject.RecommenderSystem.Framework.Metrics
{
    /// <summary>
    /// Partitions the user actions into a training and test set, where the
    /// test set is made of the most recent N ratings.
    /// </summary>
    /// <typeparam name="TUserActionType">The type of the user action type.</typeparam>
    public class MostRecentAllButNTestSet<TUserActionType> : AllButNTestSetBase<TUserActionType>
        where TUserActionType : UserAction
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="MostRecentAllButNTestSet&lt;TUserActionType&gt;"/> class.
        /// </summary>
        /// <param name="recommender">The recommender.</param>
        /// <param name="testSetSizeN">The test set size N.</param>
        public MostRecentAllButNTestSet(Recommender recommender, int testSetSizeN)
            : base(recommender, testSetSizeN)
        {
        }

        /// <summary>
        /// Gets the training set from user feedback.
        /// </summary>
        /// <param name="feedbackItems">The feedback items.</param>
        /// <param name="toRemove">To remove.</param>
        /// <param name="count">The count.</param>
        protected override void GetTrainingSetFromUserFeedback(
            Collection<TUserActionType> feedbackItems,
            Collection<TUserActionType> toRemove, 
            int count)
        {
            IEnumerable<TUserActionType> feedbacksToRemove =
                feedbackItems.OrderByDescending(fbi => fbi.Timestamp).Take(count);
            foreach (TUserActionType feedbackContainer in feedbacksToRemove)
            {
                toRemove.Add(feedbackContainer);
            }
        }

    }
}
