﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using EMIC.Design;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Framework;

namespace MyMediaProject.RecommenderSystem.Framework.Metrics
{
    /// <summary>
    /// Base class for partitioning user action sets into training and test sets.
    /// </summary>
    /// <typeparam name="TUserActionType">The type of the user action type.</typeparam>
    public abstract class TestSetQualifier<TUserActionType>
        where TUserActionType : UserAction
    {
        private Recommender recommender;

        /// <summary>
        /// Initializes a new instance of the <see cref="TestSetQualifier&lt;TUserActionType&gt;"/> class.
        /// </summary>
        /// <param name="recommender">The recommender instance for feedback removal and/or addition.</param>
        protected TestSetQualifier(Recommender recommender)
        {
            this.recommender = recommender;
        }

        /// <summary>
        /// Extracts the test set.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <param name="userFeedbacks">The user feedbacks.</param>
        /// <param name="testSet">The test set.</param>
        public abstract void ExtractTestSet(
            User userId,
            IQueryable<TUserActionType> userFeedbacks,
            Collection<TUserActionType> testSet);

        /// <summary>
        /// Gets the recommender.
        /// </summary>
        /// <value>The recommender.</value>
        protected Recommender Recommender
        {
            get { return this.recommender; }
        }
    }
}
