﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.Linq;
using System.Drawing;
using System.Text;
using System.IO;
using System.Xml;
using System.Windows.Forms;
using System.Linq;
using FacebookIntegrationComponent;
using ExternalSystemIntegration;
using MyMediaProject.RecommenderSystem.Core;

namespace FacebookTestGUI
{
    public partial class Form1 : Form
    {
        private FacebookIntegrator facebookIntegrator;
        private WebBrowserRedirectRequest lastRedirectRequest;
        List<ExternalSystemUser> fbFriends;

        private String sessionKey;
        private String authenticatedUserId;

        ExternalSystemUser extUser; // the user with Facebook
        const String GetFriendsFeatureName = "GetUsersFriends";

        /// <summary>
        /// Constructor, initializing facebookIntegrator object
        /// </summary>
        public Form1()
        {
            InitializeComponent();

            facebookIntegrator = new FacebookIntegrator(true);

            if (!facebookIntegrator.IsFeatureSupported("get_list_of_friends"))
            {
                //btnGetListOfFriends.Enabled = false;
            }

            try
            {
                ExternalSystemIntegratorResult resultObj = facebookIntegrator.PerformExternalAuthentication(new Dictionary<String, Object>());
                this.lastRedirectRequest = resultObj.GetRedirectRequestResult();
                tbLoginUrl.Text = lastRedirectRequest.Uri.ToString();
            }
            catch (ExternalSystemIntegratorException ex)
            {
                MessageBox.Show("Error: " + ex.Message);
            }

        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnDoMethodCall_Click(object sender, EventArgs e)
        {
            if (tbAuthToken.Text == null || tbAuthToken.Text.Length == 0)
            {
                MessageBox.Show("The authentication token is not set.");
            }
            else
            {
                Dictionary<string, string> cgiParams = new Dictionary<string, string>();
                cgiParams.Add("auth_token", tbAuthToken.Text);
                ExternalSystemIntegratorResult userAuthenticationResult = null;
                try
                {
                    userAuthenticationResult = facebookIntegrator.PerformOperationCall(lastRedirectRequest.MethodNameToPassCallback, cgiParams);
                }
                catch (ExternalSystemIntegratorException ex)
                {
                    MessageBox.Show("Error has occured. Incorrect configuration or entered authentication token?\n\n (Details:" + ex.ToString() + ")");
                    return;
                } 
                
                if (userAuthenticationResult.IsStringDictionaryResultType())
                {
                    Dictionary<String, String> dictUserAuthenticationResult = userAuthenticationResult.GetStringDictionaryResult();
                    if (dictUserAuthenticationResult.ContainsKey(FacebookIntegrator.CgiParameterNameSessionKey))
                    {
                        this.sessionKey = dictUserAuthenticationResult[FacebookIntegrator.CgiParameterNameSessionKey];
                    }

                    if (dictUserAuthenticationResult.ContainsKey(FacebookIntegrator.CgiParameterNameFacebookUserId))
                    {
                        this.authenticatedUserId = dictUserAuthenticationResult[FacebookIntegrator.CgiParameterNameFacebookUserId];
                    }
                }
                display_status_msg("Auth_token must have been set with a callback.");


                extUser = facebookIntegrator.GetUserProfile(authenticatedUserId, sessionKey);
                rtResult.Text = "User signed in: " + extUser.DisplayName;
                display_status_msg("External ID = " + extUser.UserExternalIdentifier);

                extUser = facebookIntegrator.AssociateExternalAndLocalUser(extUser);
                rtResult.Text = "User signed in: " + extUser.DisplayName + "\r\n" + rtResult.Text;

                btnGetListOfFriends.Enabled = false;
                if (extUser != null && extUser.User != null)
                {
                    facebookIntegrator.DisableIntegrationFeatureForUser(GetFriendsFeatureName, extUser.User);
                    VerifyFriendsFeatureAllowance();
                    //MessageBox.Show("About to enable the feature..");

                    facebookIntegrator.EnableIntegrationFeatureForUser(GetFriendsFeatureName, extUser.User);
                    VerifyFriendsFeatureAllowance();
                }

                tbNotificationText.Text = extUser.FirstName + " " + extUser.LastName
                    + " recommends you <a href='http://www.filmaxinternational.com/fichas/1262.htm'>DONKEY XOTE</a>. <br/>Enjoy <a href='http://www.mymediaproject.org/'>MyMedia</a>!";
            }
        }


        private void VerifyFriendsFeatureAllowance(){
            if (facebookIntegrator.IsFeatureAllowedByUser(GetFriendsFeatureName, extUser.User))
            {
                btnGetListOfFriends.Enabled = true;
            }
        }


        private void btnGetListOfFriends_Click(object sender, EventArgs e)
        {
            if (comboFriends != null && comboFriends.Items != null && comboFriends.Items.Count > 0)
            {
                MessageBox.Show("Friends list has been loaded already");
                return;
            }
            try
            {
                if (!facebookIntegrator.IsFeatureAllowedByUser(GetFriendsFeatureName, extUser.User))
                {
                    MessageBox.Show("The feature is blocked by user.");
                }
                fbFriends = (List<ExternalSystemUser>)facebookIntegrator.GetListOfFriends(sessionKey);
                foreach (ExternalSystemUser friend in fbFriends)
                {
                    comboFriends.Items.Add(friend.DisplayName);
                }
                display_status_msg("Number of friends loaded: " + fbFriends.Count);
            }
            catch (ExternalSystemIntegratorException ex)
            {
                MessageBox.Show("Error: " + ex.Message);
            }
        }

        private void btnSendNotification2User_Click(object sender, EventArgs e)
        {
            int idx = comboFriends.SelectedIndex;
            if (fbFriends == null || idx < 0)
            {
                MessageBox.Show("Error: User has not been chosen.");
            }
            else
            {
                string uid = fbFriends[idx].UserExternalIdentifier; // comboFriends.SelectedIndex.DisplayName ;

                MessageBox.Show("Sending Msg to a user " + uid + ":\r\n (" + tbNotificationText.Text + ")");
                List<ExternalSystemUser> toUsers = new List<ExternalSystemUser>();
                toUsers.Add(fbFriends[idx]);
                try
                {
                    facebookIntegrator.SendMessageToExternalUsers(toUsers, tbNotificationText.Text, this.sessionKey);
                    display_status_msg("The recommendation has been sent to a user: " + fbFriends[idx].FirstName + fbFriends[idx].LastName + "(" + fbFriends[idx].UserExternalIdentifier + ")");
                }
                catch (ExternalSystemIntegratorException ex)
                {
                    MessageBox.Show("Error: " + ex.Message);
                }
            }
        }

        private void display_status_msg(string msg)
        {
            lblStatusLine.Text = msg;
        }

        private void btnPublishStory_Click(object sender, EventArgs e)
        {
            try
            {
                ExternalSystemIntegratorResult resultObj = facebookIntegrator.IsApplicationAdded(this.sessionKey);
                if (!resultObj.GetBoolResult())
                {
                    MessageBox.Show("Application is not installed.");
                }
                else
                {
                    facebookIntegrator.PublishUserFavoriteInTheNewsfeed(ContentItemNameLikedTextBox.Text, this.sessionKey);
                    display_status_msg("The story with the title '" + ContentItemNameLikedTextBox.Text + "' has been published.");
                }
            }
            catch (ExternalSystemIntegratorException ex)
            {
                MessageBox.Show("Error: " + ex.Message);
            }
        }

        private void btnUserGroups_Click(object sender, EventArgs e)
        {
            try
            {
                Dictionary<String, String> groups = facebookIntegrator.GetListOfUserGroups(this.authenticatedUserId, this.sessionKey);

                if (groups != null && groups.Count > 0){
                    HashSet<ExternalGroupsManagement.ExternalGroupIdNamePair> groupIdNamePairs = new HashSet<ExternalGroupsManagement.ExternalGroupIdNamePair>();
                    rtResult.Text = "User groups:\r\n";
                    foreach (String groupIdentifier in groups.Keys)
                    {
                        ExternalGroupsManagement.ExternalGroupIdNamePair groupData = new ExternalGroupsManagement.ExternalGroupIdNamePair();
                        groupData.Name = groups[groupIdentifier];
                        groupData.ExternalIdentifier = groupIdentifier;
                        groupIdNamePairs.Add(groupData);
                        rtResult.Text += "'" + groups[groupIdentifier] + "'\r\n";
                    }
                    facebookIntegrator.EnsureUserExternalGroupMembershipsAreUpToDate(extUser, groupIdNamePairs);
                }

                display_status_msg("User groups loaded.");
            }
            catch (ExternalSystemIntegratorException ex)
            {
                MessageBox.Show("Error: " + ex.Message);
            }
        }

        private void RegisterTemplateBundleButton_Click(object sender, EventArgs e)
        {
            int iResult = facebookIntegrator.ClearAllRegisteredApplicationTemplateBundles();
            display_status_msg("Number of application template bundles deactivated: '" + iResult + "'.");

            String templateBundleId = facebookIntegrator.InitializeUserLikedItemNewsfeedTemplateBundle();
            display_status_msg("Template bundle ID: " + templateBundleId);
        }
    }
}
