// UIMAWrapper.h

#pragma once

#include "uima/api.hpp"
#include "AnnotationWrapper.h"
#include "AnnotationType.h"

using namespace uima;
using namespace System;
using namespace System::Collections::Generic;

namespace mymedia {

    /// <summary>
    /// Class that wraps around the c++ interface of Apache UIMA.
    /// </summary>
    public ref class UIMAWrapper
	{
    public:
        /// <summary>
        /// Constructor, initializes the engine
        /// </summary>
        /// <param name="descriptor">The full path to the analysis engine descriptor file.</param>
        UIMAWrapper(System::String^ descriptor);
        ~UIMAWrapper();
        !UIMAWrapper();

        /// <summary>
        /// Create annotations for a text. Annotations are returned in list. Only annotations with types that have
        /// been added through addAnnotationType are included.
        /// </summary>
        /// <param name="corpus">The text.</param>
        /// <returns>List of annotations</returns>
        LinkedList<AnnotationWrapper^>^ tag(String^ corpus);

        /// <summary>
        /// Add an annotation type to watch out for. If an analysis engine has this annotation as output,
        /// these annotations will be returned by the tag function.
        /// </summary>
        /// <param name="annotation">The annotation type.</param>
        void addAnnotationType(AnnotationType^ annotation);

        /// <summary>
        /// Shutdown engine.
        /// </summary>
        void finish();

    private:
        LinkedList<AnnotationWrapper^>^ processInternal(UnicodeString * ucs, AnalysisEngine * pEngine, CAS* tcas);
        UnicodeString* stringToUnicode(String^ string);
        AnalysisEngine * pEngine;
        TyErrorId utErrorId;               // Variable to store return codes
        ErrorInfo* errorInfo;             // Variable to stored detailed error info
        CAS* tcas;
        IList<AnnotationType^>^ annotations;
	};
}
