package nl.telin.mymedia.uima.annotator;

import nl.telin.mymedia.apolda.Apolda;

import org.apache.uima.analysis_engine.ResultSpecification;
import org.apache.uima.analysis_engine.annotator.AnnotatorConfigurationException;
import org.apache.uima.analysis_engine.annotator.AnnotatorContext;
import org.apache.uima.analysis_engine.annotator.AnnotatorContextException;
import org.apache.uima.analysis_engine.annotator.AnnotatorInitializationException;
import org.apache.uima.analysis_engine.annotator.AnnotatorProcessException;
import org.apache.uima.analysis_engine.annotator.JTextAnnotator_ImplBase;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.TypeSystem;
import org.apache.uima.jcas.JCas;


/**
 * @author Wout.Slakhorst
 * Create OntologyAnnotations based on an ontology
 */
public class OntologyAnnotator extends JTextAnnotator_ImplBase {

	//private Logger logger;
	//private Model model;
	Apolda annotator;

	//private static final String TT_ANNOTATION_NAME = "nl.telin.mymedia.uima.TreeTaggerAnnotation";
	private static final String TOKEN_ANNOTATION_NAME = "org.apache.uima.TokenAnnotation";

	//private Type ttAnnotation;
	private Type tokenAnnotation;
	private Feature lemmaFeature;
	
	@Override
	/**
	 * @inheritDoc
	 * config parameters:
	 * ONTOLOGY - ontology path
	 * PREF_LABEL - ontology uri for prefered label 
	 * ALT_LABEL - ontology uri for alternative label
	 * LANGUAGE - language
	 */
	public void initialize(AnnotatorContext context)
			throws AnnotatorInitializationException,
			AnnotatorConfigurationException {
		super.initialize(context);

		try {
			//logger = getContext().getLogger();

			String OntURI = (String) getContext().getConfigParameterValue("ONTOLOGY");
			String prefLabel = (String) getContext().getConfigParameterValue("PREF_LABEL");
			String altLabel = (String) getContext().getConfigParameterValue("ALT_LABEL");
			String language = (String) getContext().getConfigParameterValue("LANGUAGE");
				
			annotator = new Apolda();
			annotator.initialize(OntURI,prefLabel,altLabel,language);

		} catch (AnnotatorContextException e) {
			throw new AnnotatorConfigurationException(e);
		}

	}

	@Override
	/**
	 * @inheritDoc
	 */
	public void typeSystemInit(TypeSystem aTypeSystem)
			throws AnnotatorInitializationException,
			AnnotatorConfigurationException {

		// initialize cas token type
		//this.ttAnnotation = aTypeSystem.getType(TT_ANNOTATION_NAME);		
		this.tokenAnnotation = aTypeSystem.getType(TOKEN_ANNOTATION_NAME);
	}

	//@Override
	/**
	 * @inheritDoc
	 */
	public void process(JCas jCas, ResultSpecification arg1)
			throws AnnotatorProcessException {
		//TODO lemmaFeature zou iets moten zijn dat 'lemma' uit de parameters meegeeft.
		annotator.analyze(jCas, lemmaFeature,tokenAnnotation);
	}

}
