package nl.telin.mymedia.uima.annotator;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import nl.telin.mymedia.uima.annotation.DeletedAnnotation;
import nl.telin.mymedia.uima.annotation.GivenName;
import nl.telin.mymedia.uima.annotation.LocationAnnotation;
import nl.telin.mymedia.uima.annotation.LocationName;
import nl.telin.mymedia.uima.annotation.OntologyAnnotation;
import nl.telin.mymedia.uima.annotation.PersonAnnotation;
import nl.telin.mymedia.uima.annotation.Surname;

import org.apache.uima.TokenAnnotation;
import org.apache.uima.analysis_engine.ResultSpecification;
import org.apache.uima.analysis_engine.annotator.AnnotatorConfigurationException;
import org.apache.uima.analysis_engine.annotator.AnnotatorContext;
import org.apache.uima.analysis_engine.annotator.AnnotatorContextException;
import org.apache.uima.analysis_engine.annotator.AnnotatorInitializationException;
import org.apache.uima.analysis_engine.annotator.AnnotatorProcessException;
import org.apache.uima.analysis_engine.annotator.JTextAnnotator_ImplBase;
import org.apache.uima.cas.CASException;
import org.apache.uima.cas.FSIterator;
import org.apache.uima.jcas.JCas;
import org.apache.uima.jcas.tcas.Annotation;
import org.apache.uima.util.Level;
import org.apache.uima.util.Logger;

/**
 * @author Wout.Slakhorst
 * Creates PersonAnnotations based on rules. 
 */
public class RuleAnnotator extends JTextAnnotator_ImplBase {

	private Logger logger;

	// private List<Set<Annotation>> annotationList;
	private Set<Annotation> annotationList[];

	/**
	 * @inheritDoc
	 */
	public void initialize(AnnotatorContext context)
			throws AnnotatorInitializationException,
			AnnotatorConfigurationException {
		super.initialize(context);

		try {
			logger = getContext().getLogger();
		} catch (AnnotatorContextException e) {
			throw new AnnotatorConfigurationException(e);

		}

	}

	/**
	 * @inheritDoc
	 * The following rules are described as pairs (occurence order in text)
	 * Rules: 
	 * GivenNameAnnotation, SurNameAnnottion => PersonAnnotation
	 * GivenNameAnnotation, TokenAnnotation[NNP] => PersonAnnotation
	 * TokenAnnotation[NNP], TokenAnnotation[NNP] => PersonAnnotation
	 * TokenAnnotation[TO, IN], LocationAnnotation => PersonAnnotation
	 */
	public void process(JCas cas, ResultSpecification resultSpec)
			throws AnnotatorProcessException {
		FSIterator ttIterator = cas.getCas().getAnnotationIndex().iterator();

		annotationList = new HashSet[cas.getDocumentText().length()];
		if(annotationList.length > 0) {

			while (ttIterator.hasNext()) {
				Annotation a = (Annotation) ttIterator.next();


				// already a set present?
				Set<Annotation> s = annotationList[a.getBegin()];
				if (s == null) {
					s = new HashSet<Annotation>();
				}
				s.add(a);
				annotationList[a.getBegin()] = s;
			}

			doIntelligentStuff(annotationList);
		}
	}

	private void doIntelligentStuff(Set<Annotation>[] annotationList2) {

		// previous tokens for more complex rules;
		Map<String, Annotation> sPrev = new HashMap<String, Annotation>();
		for (int i = 0; i < annotationList.length; i++) {
			Set<Annotation> s = annotationList[i];

			// is this a whitespace token, then skip
			if (s != null && !isWhiteSpaceToken(s)) {
				// the standard set of annotations we're interested in.
				TokenAnnotation token = null;
				OntologyAnnotation ontology = null;
				Surname surName = null;
				GivenName givenName = null;
				LocationName locationName = null;

				if (s != null) {
					for (Annotation a : s) {
						if (a instanceof TokenAnnotation) {
							token = (TokenAnnotation) a;
						} else if (a instanceof OntologyAnnotation) {
							ontology = (OntologyAnnotation) a;
						} else if (a instanceof Surname) {
							surName = (Surname) a;
						} else if (a instanceof GivenName) {
							givenName = (GivenName) a;
						} else if (a instanceof LocationName) {
							locationName = (LocationName) a;
						}
					}

					// rules

					// remove ontology token when previous token is a firstname.
					// todo add NameAnnotation
					if (ontology != null && sPrev.get("givenName") != null
							&& "NNP".equals(token.getPos())) {
						try {
							DeletedAnnotation newAnnot = new DeletedAnnotation(
									ontology.getCAS().getJCas(), ontology
											.getBegin(), ontology.getEnd());
							newAnnot.addToIndexes();
							// PersonAnnotation.xml enables this annotation type
							// don't forget to add the annotation type to the
							// analysis engine descriptor
							// and annotator.
							PersonAnnotation pAnnot = new PersonAnnotation(
									ontology.getCAS().getJCas(), sPrev.get(
											"givenName").getBegin(), ontology
											.getEnd());
							pAnnot.addToIndexes();
							ontology.removeFromIndexes();
						} catch (CASException e) {
							logger.log(Level.SEVERE,
									"Error creating deletedAnnotation", e);
						}						
					} else if (surName != null && sPrev.get("givenName") != null) {
						try {
							PersonAnnotation pAnnot = new PersonAnnotation(
									surName.getCAS().getJCas(), sPrev.get(
											"givenName").getBegin(), surName
											.getEnd());
							pAnnot.addToIndexes();
						} catch (CASException e) {
							logger.log(Level.SEVERE,
									"Error creating deletedAnnotation", e);
						}						
					} else if (token!=null && "NNP".equals(token.getPos())
							&& sPrev.get("givenName") != null) {
						try {
							PersonAnnotation pAnnot = new PersonAnnotation(
									token.getCAS().getJCas(), sPrev.get(
											"givenName").getBegin(), token
											.getEnd());
							pAnnot.addToIndexes();
						} catch (CASException e) {
							logger.log(Level.SEVERE,
									"Error creating deletedAnnotation", e);
						}						
					}

					// BUGGED
					// Changed CW 20081113
					else if (token != null && "NNP".equals(token.getPos()) 
							&& sPrev.get("token") != null
							&& "NNP".equals(((TokenAnnotation) sPrev.get("token")).getPos())
							&& givenName == null
							&& locationName == null
							&& ((TokenAnnotation) sPrev.get("token")).getLemma().isEmpty() //CW 20081113
							&& token.getLemma().isEmpty()) { //CW 20081113
						try {
							Annotation prev = sPrev.get("token");
							PersonAnnotation pAnnot = new PersonAnnotation(
									token.getCAS().getJCas(), prev.getBegin(),
									token.getEnd());
							pAnnot.addToIndexes();
						} catch (CASException e) {
							logger.log(Level.SEVERE,
									"Error creating deletedAnnotation", e);
						}
						//continue;
						
					} else if (locationName != null 
							&& sPrev.get("token") != null
							&& ("TO".equals(((TokenAnnotation) sPrev.get("token")).getPos())
									|| "IN".equals(((TokenAnnotation) sPrev.get("token")).getPos())
									)
						){
						try {
							//Annotation prev = sPrev.get("token");
							LocationAnnotation lAnnot = new LocationAnnotation(
									token.getCAS().getJCas(), token.getBegin(),
									token.getEnd());
							lAnnot.addToIndexes();
						} catch (CASException e) {
							logger.log(Level.SEVERE,
									"Error creating deletedAnnotation", e);
						}
					
					}
				}

				// clear sPrev and fill with current values
				sPrev.clear();
				sPrev.put("token", token);
				sPrev.put("ontology", ontology);
				sPrev.put("surName", surName);
				sPrev.put("givenName", givenName);
				sPrev.put("locationName", locationName);
			}
		}
	}
	
	private boolean isWhiteSpaceToken(Set<Annotation> annotations) {
		// only size == 1 can be whitespace
		if (annotations.size() == 1) {
			Annotation a = annotations.iterator().next();
			if (a instanceof TokenAnnotation) {
				TokenAnnotation token = (TokenAnnotation) a;
				if ("PUNC".equals(token.getTokentype())
						&& (token.getLemma() == null || token.getLemma().trim().isEmpty())) {
					return true;
				}
			}

		}
		return false;
	}

}
