package nl.telin.mymedia.uima.annotator;

import java.text.BreakIterator;
import java.text.ParsePosition;
import java.util.Locale;

import org.apache.uima.SentenceAnnotation;
import org.apache.uima.TokenAnnotation;
import org.apache.uima.analysis_component.JCasAnnotator_ImplBase;
import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.jcas.JCas;
import org.apache.uima.jcas.tcas.Annotation;

public class TokenAnnotator extends JCasAnnotator_ImplBase {

	static abstract class Maker {
		abstract Annotation newAnnotation(JCas jcas, int start, int end);
	}

	JCas jcas;

	String input;

	ParsePosition pp = new ParsePosition(0);

	// ****************************************
	// * Static vars holding break iterators
	// ****************************************
	static final BreakIterator sentenceBreak = BreakIterator
			.getSentenceInstance(Locale.US);

	static final BreakIterator wordBreak = BreakIterator
			.getWordInstance(Locale.US);

	// *********************************************
	// * function pointers for new instances *
	// *********************************************
	static final Maker sentenceAnnotationMaker = new Maker() {
		Annotation newAnnotation(JCas jcas, int start, int end) {
			return new SentenceAnnotation(jcas, start, end);
		}
	};

	static final Maker tokenAnnotationMaker = new Maker() {
		Annotation newAnnotation(JCas jcas, int start, int end) {
			return new TokenAnnotation(jcas, start, end);
		}
	};

	// *************************************************************
	// * process *
	// *************************************************************
	public void process(JCas aJCas) throws AnalysisEngineProcessException {
		jcas = aJCas;
		input = jcas.getDocumentText();

		// Create Annotations
		makeAnnotations(sentenceAnnotationMaker, sentenceBreak);
		makeAnnotations(tokenAnnotationMaker, wordBreak);
	}

	// *************************************************************
	// * Helper Methods *
	// *************************************************************
	void makeAnnotations(Maker m, BreakIterator b) {
		b.setText(input);
		for (int end = b.next(), start = b.first(); end != BreakIterator.DONE; start = end, end = b
				.next()) {
			// eliminate all-whitespace tokens
			boolean isWhitespace = true;
			for (int i = start; i < end; i++) {
				if (!Character.isWhitespace(input.charAt(i))) {
					isWhitespace = false;
					break;
				}
			}
			if (!isWhitespace) {
				m.newAnnotation(jcas, start, end).addToIndexes();
			}
		}
	}
}