package nl.telin.mymedia.uima.annotator;

import java.io.IOException;
import java.text.BreakIterator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import nl.telin.mymedia.processor.ProcessorException;
import nl.telin.mymedia.processor.ProcessorInitializeException;
import nl.telin.mymedia.processor.token.RegExpTokenizer;
import nl.telin.mymedia.processor.token.TextToken;

import org.apache.uima.SentenceAnnotation;
import org.apache.uima.TokenAnnotation;
import org.apache.uima.analysis_engine.ResultSpecification;
import org.apache.uima.analysis_engine.annotator.AnnotatorConfigurationException;
import org.apache.uima.analysis_engine.annotator.AnnotatorContext;
import org.apache.uima.analysis_engine.annotator.AnnotatorContextException;
import org.apache.uima.analysis_engine.annotator.AnnotatorInitializationException;
import org.apache.uima.analysis_engine.annotator.AnnotatorProcessException;
import org.apache.uima.analysis_engine.annotator.JTextAnnotator_ImplBase;
import org.apache.uima.cas.TypeSystem;
import org.apache.uima.jcas.JCas;
import org.apache.uima.jcas.tcas.Annotation;
import org.apache.uima.resource.ResourceProcessException;
import org.apache.uima.util.Logger;
import org.apache.uima.util.ProcessTrace;

public class Tokenizer extends JTextAnnotator_ImplBase {

	static abstract class Maker {
		abstract Annotation newAnnotation(JCas jcas, int start, int end);
	}
	
	private static final String TT_ANNOTATION_NAME = "org.apache.uima.TokenAnnotation";

	
	private RegExpTokenizer tokenizer;
	
	private Logger logger;
	
	private String text;
	private JCas jcas;

	public void initialize(AnnotatorContext context)
			throws AnnotatorInitializationException,
			AnnotatorConfigurationException {
		super.initialize(context);

		try {
			logger = getContext().getLogger();

			tokenizer = new RegExpTokenizer();
			
			Map<String, String> params = new HashMap<String, String>();
			params.put(RegExpTokenizer.PARAM_SENTENCEEXPR, (String) getContext().getConfigParameterValue("sentRegExp"));
			params.put(RegExpTokenizer.PARAM_WORDEXPRFILE, (String) getContext().getConfigParameterValue("regExpFile"));
			
			tokenizer.initialize(params);
			

		} catch (AnnotatorContextException e) {
			throw new AnnotatorConfigurationException(e);
		} catch (ProcessorInitializeException e) {
			throw new AnnotatorConfigurationException(e);
		}

	}

	public void typeSystemInit(TypeSystem aTypeSystem)
			throws AnnotatorInitializationException,
			AnnotatorConfigurationException {

		// initialize cas token type
		//this.ttAnnotation = aTypeSystem.getType(TT_ANNOTATION_NAME);

	}

	public void process(JCas cas, ResultSpecification res)
			throws AnnotatorProcessException {
		
		text = cas.getDocumentText();
		jcas = cas;
		
		try {
			List<List<TextToken>> tokens = tokenizer.process(text);
			
			for (List<TextToken> sentence : tokens) {
				if (!sentence.isEmpty()) {
					SentenceAnnotation token = new SentenceAnnotation(cas, sentence.get(0).getBegin(), sentence.get(sentence.size()-1).getEnd());
					token.addToIndexes();
					
					for (TextToken word : sentence) {
						TokenAnnotation wt = new TokenAnnotation(cas, word.getBegin(), word.getEnd());
						wt.setTokentype(word.getType());
						wt.addToIndexes();
					}
				}
			}
		} catch (ProcessorException e) {
			//throw new AnnotatorProcessException(e);
			System.out.println("Can't process: " + text);			
		}
		
	}
	
	//@Override
	public void destroy() {
		System.out.println("Finalizing RegExpTokenizer");
	}
	
	void makeAnnotations(Maker m, BreakIterator b) {
		b.setText(text);
		for (int end = b.next(), start = b.first(); end != BreakIterator.DONE; start = end, end = b
				.next()) {
			// eliminate all-whitespace tokens
			boolean isWhitespace = true;
			for (int i = start; i < end; i++) {
				if (!Character.isWhitespace(text.charAt(i))) {
					isWhitespace = false;
					break;
				}
			}
			if (!isWhitespace) {
				m.newAnnotation(jcas, start, end).addToIndexes();
			}
		}
	}		
}
