package nl.telin.mymedia.uima.multiplier;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.uima.UimaContext;
import org.apache.uima.analysis_component.JCasMultiplier_ImplBase;
import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.cas.AbstractCas;
import org.apache.uima.jcas.JCas;
import org.apache.uima.resource.ResourceInitializationException;

/**
 * @author Wout.Slakhorst
 * UIMA multiplier for loading movie describtions from the mymedia recommender framework database
 */
public class SQLServerMultiplier extends JCasMultiplier_ImplBase {

	private String connectionString;
	private String tableName;
	private String columnName;
	private Connection conn;
	private Statement st;
	private ResultSet rs;
	
	private int maxResults = 10000;
	private int currentcount = 0;
	private static boolean ranOnce = false;
	
	//@Override
	/**
	 * @inheritDoc
	 * Config parameters:
	 * connectionString - sqlserver connection string
	 * tableName - table name
	 * columnName - column name
	 */
	public void initialize(UimaContext aContext) throws ResourceInitializationException {
		super.initialize(aContext);
		
		connectionString = (String) aContext.getConfigParameterValue("connectionString");
		tableName = (String) aContext.getConfigParameterValue("tableName");
		columnName = (String) aContext.getConfigParameterValue("columnName");
				

		// init db
		try {
			Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");					
		} catch (ClassNotFoundException e) {
			throw new ResourceInitializationException(e);
		}
		
		
	}

	private boolean dbIsOpen() {
		return conn != null;
	}
	
	private void openDb() throws SQLException {
		conn = DriverManager.getConnection(connectionString);
		
		
		st = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
		rs = st.executeQuery("select " + columnName + " from " + tableName);
		
	}

	//@Override
	/**
	 * @inheritDoc
	 */
	public boolean hasNext() throws AnalysisEngineProcessException {
		boolean hasNext = false;
		try {
			if (rs != null && !rs.isLast()) {
				hasNext = true;
			}
		} catch (SQLException e) {
			throw new AnalysisEngineProcessException(e);
		}
		
		if (currentcount > maxResults) {
			hasNext = false;			
		}
		
		
		return hasNext;
	}

	@Override
	/**
	 * @inheritDoc
	 */
	public void process(JCas cas) throws AnalysisEngineProcessException {
		if (!ranOnce) {
			try {
				if (!dbIsOpen()) {
					openDb();
				}
			} catch (SQLException e) {
				throw new AnalysisEngineProcessException(e);
			}
			
			ranOnce = true;
		}
		
	}

	//@Override
	/**
	 * @inheritDoc
	 */
	public AbstractCas next() throws AnalysisEngineProcessException {
		JCas jcas = getEmptyJCas();		
		currentcount++;
		try {
			rs.next();
			jcas.setDocumentText(rs.getString(1));
			return jcas;
		} catch (SQLException e) {
			throw new AnalysisEngineProcessException(e);
		}
	}

	//@Override
	/**
	 * @inheritDoc
	 */
	public void collectionProcessComplete()
			throws AnalysisEngineProcessException {
		super.collectionProcessComplete();
		try {
			rs.close();
			st.close();
			conn.close();
		} catch (SQLException e) {
			throw new AnalysisEngineProcessException(e);
		}
	}
	

}
