package nl.telin.mymedia.uima.trainer;

import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import nl.telin.mymedia.db.Lemma;
import nl.telin.mymedia.db.Stats;
import nl.telin.mymedia.db.util.HibernateUtil;

import org.apache.uima.TokenAnnotation;
import org.apache.uima.cas.CAS;
import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Type;
import org.apache.uima.collection.CasConsumer_ImplBase;
import org.apache.uima.resource.ResourceInitializationException;
import org.apache.uima.resource.ResourceProcessException;
import org.apache.uima.util.ProcessTrace;

/**
 * @author Wout.Slakhorst
 * tfidf trainer, this CasConsumer counts tokens and updates the docCount in the database *  
 */
public class TfIdfTrainer extends CasConsumer_ImplBase {

	private static int count;
	
	private static final String TOKEN_ANNOTATION_NAME = "org.apache.uima.TokenAnnotation";
	private Type tokenAnnotation;
	private Map<String, Lemma> lemmas;
	
	private static Set<String> semPos;
	
	static {
		semPos = new HashSet<String>();
		semPos.add("NN");
		semPos.add("NNP");
		semPos.add("JJ");
		semPos.add("VBG");
		semPos.add("VBD");
		semPos.add("VBN");
		semPos.add("VBP");
		semPos.add("VB");
		semPos.add("VBZ");
		semPos.add("FW");
		semPos.add("JJR");
		semPos.add("JJS");
		semPos.add("NNS");
		semPos.add("NNPS");
		semPos.add("NP");
		semPos.add("NPS");		
	}
	
	private static Set<String> auxList;
	
	static {
		auxList = new HashSet<String>();
		auxList.add("do");
		auxList.add("be");
		auxList.add("have");
		auxList.add("will");
		auxList.add("shall");
		auxList.add("can");
		auxList.add("may");
		auxList.add("go");
		auxList.add("make");
		auxList.add("let");
		auxList.add("try");
		auxList.add("get");
	}
	
	
	//@Override
	/**
	 * @inheritDoc
	 * empties the database!
	 */
	public void initialize() throws ResourceInitializationException {
		
		// clear db
		HibernateUtil.removeAllInfGain();
		HibernateUtil.removeAll();		
		Stats stat = HibernateUtil.findDocCount();
		if (stat == null) {
			stat = new Stats("documents", 0);
		}
		stat.setValue(0);
		HibernateUtil.updateDocCount(stat);
	}



	//@Override
	/**
	 * @inheritDoc
	 */
	public void processCas(CAS acas) throws ResourceProcessException {
		this.tokenAnnotation = acas.getTypeSystem().getType(TOKEN_ANNOTATION_NAME);		
		FSIterator tokenIterator = acas.getAnnotationIndex(this.tokenAnnotation).iterator();
		
		
		
		lemmas = new HashMap<String, Lemma>();
		
		int tokens = 0;
		while (tokenIterator.hasNext()) {
			TokenAnnotation currentToken = (TokenAnnotation)tokenIterator.next();
			
			if ("WORD".equals(currentToken.getTokentype())
					&& currentToken.getCoveredText().length() > 1
					&& semanticLemma(currentToken)
					) {
			
				Lemma lemma = findExistingLemma(currentToken);
				lemma.setFreq(lemma.getFreq() + 1);
				
				tokens++;
			}
		}
		
		// hoog statistieken op
		Stats stat = HibernateUtil.findDocCount();
		stat.setValue(stat.getValue() + 1);
		HibernateUtil.updateDocCount(stat);
		
		// hoog docCount op
		for(Lemma l : lemmas.values()) {
			l.setDocs(l.getDocs() + 1);
			HibernateUtil.saveLemma(l);			
		}
		
		//System.out.println("Processed: " + stat.getValue());
		//HibernateUtil.printTimeUsage();		
		
	}	
	
	
	
	private boolean semanticLemma(TokenAnnotation currentToken) {
		String POS = currentToken.getPos();
		return (semPos.contains(POS) && !auxVerb(currentToken));
	}


	private boolean auxVerb(TokenAnnotation token) {
		boolean rc = false;
		String POS = token.getPos();
		String lemma = token.getLemma();
		if(auxList.contains(lemma)) {
			rc = true;
		}	
		return rc;
	}



	//@Override
	/**
	 * @inheritDoc
	 */
	public void collectionProcessComplete(ProcessTrace arg0)
			throws ResourceProcessException, IOException {
		// filter common words
		System.out.println(HibernateUtil.getLemmaCount());
		HibernateUtil.filterCommonLemma();
		HibernateUtil.filterSpecificLemma();
		System.out.println(HibernateUtil.getLemmaCount());
	}



	private Lemma findExistingLemma(TokenAnnotation token) {
		String lemmaString = token.getLemma().toLowerCase();
		String normalString = token.getCoveredText().toLowerCase();
		String theString = null;
		
		Lemma lt = lemmas.get(lemmaString);
		Lemma st = lemmas.get(normalString);
		
		Lemma lemma = null;
		
		if (lt != null) {
			lemma = lt;
			theString = lemmaString;
		} else if (st != null) {
			lemma = st;
			theString = normalString;
		} else { // first occurence of this token in this text
			lt = HibernateUtil.findLemma(lemmaString);
			st = HibernateUtil.findLemma(normalString);
			
			if (lt != null) {
				lemma = lt;
				theString = lemmaString;
			} else if (st != null) {
				lemma = st;
				theString = normalString;
			} else { // first occurence ever
				if ("".equals(lemmaString) || lemmaString == null) {
					theString = normalString;
				} else {
					theString = lemmaString;
				}
				
				lemma = new Lemma(theString, 0, 0);
			}
		}
		
		HibernateUtil.saveLemma(lemma); // for id
		lemmas.put(theString, lemma);		
		return lemma;		
	}	
}
