package nl.telin.mymedia.ontology;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.apache.uima.analysis_engine.annotator.AnnotatorProcessException;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Selector;
import com.hp.hpl.jena.rdf.model.SimpleSelector;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

/**
 * This class tries to add an altLAbelSg to the ontology based on prefLabel and
 * altLabel. This is necessary because of the fact that most concepts are plural
 * and need to be singular as well.
 * 
 * @author Wout.Slakhorst
 * 
 */
public class OntologyStemmer {

	private Model model;
	private String currentLemma;

	public OntologyStemmer() {

	}

	public void loadModel(String path) throws IOException {
		try {
			InputStream stream = new FileInputStream(path);

			// load jena
			model = ModelFactory.createDefaultModel();
			model.read(stream, "");
		} catch (IOException e) {
			throw e;
		}
	}

	public void saveModel(String path) throws IOException {
		try {
			OutputStream stream = new FileOutputStream(path);

			model.write(stream);
		} catch (IOException e) {
			throw e;
		}
	}

	private void addAltLabelSg(Selector query) {
		StmtIterator iter = model.listStatements(query);
		while (iter.hasNext()) {
			Statement stmt = iter.nextStatement();
			int label = 0;

			// split words
			String[] split = stmt.getObject().toString().split("\\(");
			String left = split[0];
			String right = "";
			if (split.length > 1) { 
				right = "(" + split[1];
			}
			
			split = left.trim().split(" ");
			label = split.length - 1;			

			// enhance
			if ("s".equals(split[label].substring(split[label].length()-1))) {
				System.out.println(split[label]);
				String stem = stem(split[label]);
				if (stem!= null) {
					split[label] = stem;
				}
				System.out.println(split[label]);
	
				// recombine entire label
				String altLabel = "";
				for (String s : split) {
					altLabel = altLabel.concat(s).concat(" ");
				}
				altLabel = altLabel.concat(right);
				altLabel = altLabel.trim();
				
				
				model.add(model.createStatement(stmt.getSubject(), model
						.createProperty("http://www.w3.org/2004/02/skos/core#",
								"altLabelSg"), altLabel));
			}
		}
	}
	
	private String stem(String word) {
		try {
			String ttPath = "D:/TreeTagger/bin/tree-tagger.exe";
			String ttLib = "D:/TreeTagger/lib/english.par";
			String ttArg = "-token -lemma -sgml -no-unknown";
			
			Runtime runtime = Runtime.getRuntime();
			Process p = runtime.exec(ttPath + " " + ttLib + " " + ttArg);
	
			currentLemma = null;
			
			BufferedReader input = new BufferedReader(new InputStreamReader(p
					.getInputStream()));
			
			PrintWriter output = new PrintWriter(new BufferedOutputStream(p.getOutputStream()));
						
			new Thread(new TaggerOutput(input)).start();
			
			// add to Queue
			output.write(word);
			output.write("\n");
			output.write(".");
			output.flush();
			output.close();
			p.waitFor();
			
			return currentLemma;
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		return null;
	}

	public void processAll() {
		// find all prefLabel
		Selector query = new SimpleSelector(null, model.createProperty(
				"http://www.w3.org/2004/02/skos/core#", "prefLabel"),
				(RDFNode) null);
		addAltLabelSg(query);

		// find all altLabel
		query = new SimpleSelector(null, model.createProperty(
				"http://www.w3.org/2004/02/skos/core#", "altLabel"),
				(RDFNode) null);
		addAltLabelSg(query);
	}

	public static void main(String args[]) throws Exception {
		if (args.length < 2) {
			printUsage();
			System.exit(1);
		}

		OntologyStemmer stemmer = new OntologyStemmer();
		stemmer.loadModel(args[0]);
		stemmer.processAll();
		stemmer.saveModel(args[1]);
	}

	private static void printUsage() {
		System.out.println("Usage: java OntologyStemmer [input] [output]");

	}
	
	/**
	 * Class for reading TreeTagger output
	 * @author Wout.Slakhorst
	 *
	 */
	private class TaggerOutput implements Runnable {

		BufferedReader reader;

		public TaggerOutput(BufferedReader reader) {
			this.reader = reader;
		}

		//@Override
		public void run() {
			try {
				String line = reader.readLine();
				String[] info = line.split("\t");
				currentLemma = info[2];				
			} catch (IOException e) {
				throw new RuntimeException(e);
				// exit prog
			}
		}
	}

}
