package nl.telin.mymedia.tvanytime;

import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

/**
 * @author Wout.Slakhorst
 * Extracts comments from TvAnytime
 */
public class TvAnytimeCommentExtractor {
	
	private String inputFile;
	private String outputDir;
	
	private Map<String, String> titles;
	private Map<String, List<String>> comments;
	
	public TvAnytimeCommentExtractor(String inputFile, String outputDir) {
		this.inputFile = inputFile;
		this.outputDir = outputDir;
		
		titles = new HashMap<String, String>();
		comments = new HashMap<String, List<String>>();
	}
	
	public void run() {
		try {
			FileInputStream fileInputStream = 
		        new FileInputStream(inputFile);
		    XMLStreamReader xmlStreamReader =
		        XMLInputFactory.newInstance().
		            createXMLStreamReader(fileInputStream);
		    
		    String docName = null;
		    String id = null;
		    //String elementName = null;
		    
		    
		    while (xmlStreamReader.hasNext()) {
		    	int code = xmlStreamReader.next();
		    	switch (code) {
		    	case XMLStreamConstants.START_ELEMENT:
		    		if(xmlStreamReader.getLocalName().equals("Title")){
		    			docName = xmlStreamReader.getElementText().replace("?", "").replace("\\", "").replace("/","").replace(" ", "_");
		    			titles.put(id, docName);
		    			comments.put(id, new ArrayList<String>());
		    		} else if (xmlStreamReader.getLocalName().equals("ProgramInformation")) {
		    			id = xmlStreamReader.getAttributeValue(0);
		    		} else if (xmlStreamReader.getLocalName().equals("UserReview")) {
		    			id = xmlStreamReader.getAttributeValue(0);
		    		} else if (xmlStreamReader.getLocalName().equals("FreeTextReview")) {
		    			comments.get(id).add(xmlStreamReader.getElementText());
		    		}
		    		break;	
		    	}
		    }
		    
		    int count = 0;
		    FileWriter writer = null;
		    for(String docId : comments.keySet()) {
		    	String title = titles.get(docId);
		    	String filename = outputDir + "/Comment_" + title + "_" + count++ + ".txt";
		    	if (comments.get(docId).size() > 0) {
	    			writer = new FileWriter(filename);
	    			for(String comment : comments.get(docId)) {
	    				writer.write(comment);
	    				writer.write("\n");
	    			}
	    			writer.close();
			    }
		    }
		    
		} catch (IOException e) {
			e.printStackTrace();
		} catch (XMLStreamException e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String args[]) throws Exception {
		if (args.length < 2) {
			printUsage();
			System.exit(1);
		}

		TvAnytimeCommentExtractor ext = new TvAnytimeCommentExtractor(args[0], args[1]);
		ext.run();
	}

	private static void printUsage() {
		System.out.println("Usage: java TvAnytimeCommentExtractor [input.xml] [output_dir]");

	}
}
