﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Framework;
using MyMediaProject.RecommenderSystem.Framework.Databases;
using MyMediaProject.RecommenderSystem.Framework.EntityRelationAlgorithm;
using MyMediaProject.RecommenderSystem.Core.EntityRelationshipEngine;
using MyMediaProject.RecommenderSystem.Algorithms;

namespace MovieLens
{
    public class MovieLensRecommender : Recommender
    {
        
        private const string explicitEngineDescription = "Returns the rate a user previously (explicitly) assigned to an item.";
        private const string explicitEngineGuidString = "82A2E18A-89BE-48f4-9A11-B701298DCE51";
        
        private const string userLogonName = "MovieLensRecommenderUser";

        private const string programIdGuid = "{A53D4502-5DA1-4dfe-81FF-CADF03588064}";
        
        private const string flowIdGuid = "6F05137E-F9A4-4ee1-8583-AE6C1543E7E6";

        private FlowId flowId = new FlowId(new Guid(flowIdGuid));
          
        private const string programIdFriendlyName = "MovieLensRecommender";
        private static ProgramId programId = new ProgramId(new Guid(programIdGuid), programIdFriendlyName);

        private Flow recommenderFlow;

        private MovieLensDataFactory factory;
        private EntityRelationEngine matrixFactorizationEngine;
//        private AdaptedPageRankRecommender adaptedPageRankRecommender;

        private bool intitialized;

        public override void Initialize()
        {
            if (!intitialized)
            {
                base.ProgramId = programId;

                if (!base.DatabaseExists())
                {
                    this.RestoreDatabase();
                }

                base.Initialize();

                {
                    //Do MovieLens specific initialization
                    this.factory = new MovieLensDataFactory();

                    recommenderFlow = new Flow(flowId);

                    matrixFactorizationEngine = new PredictRatingEngine(new MatrixFactorization());
                    recommenderFlow.AddEngine(matrixFactorizationEngine);

                    recommenderFlow.Hybridizer = new SingleEngineHybridizer();
                    base.RegisterFlow(recommenderFlow);
                    base.RegisterCatalogFactory(this.factory);
                    factory.ImportData();
                    intitialized = true;
                }
            }
        }

        public FlowId DefaultFlowId
        {
            get
            {
                return this.flowId;
            }
        }

        public void Train()
        {
            this.matrixFactorizationEngine.Train();
        }

        private void RestoreDatabase()
        {
            try
            {
                DatabaseHelper.RestoreRecommenderDatabase(this);
            }
            catch (Exception)
            {                
            }
        }
    }
}
