﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using MyMediaProject.RecommenderSystem.Core;
using MyMediaProject.RecommenderSystem.Framework;
using MovieLens;

namespace RecommenderFieldTrialSampleWCF
{
    // NOTE: If you change the class name "Service1" here, you must also update the reference to "Service1" in Web.config and in the associated .svc file.
    /// <summary>
    /// A WCF service class to expose business-level recommender functionality.  This class provides a simple example of 
    /// how to map HTTP addressible web services to the MyMediaProject Recommender system.
    /// </summary>
    [ServiceContract]
    public class SampleRecommenderWebService
    {
        /// <summary>
        /// Gets the ratings for user.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <returns></returns>
        [OperationContract]
        public UserRating[] GetRatingsForUser(int userId)
        {
            MovieLensRecommender recommender = RecommenderSingleton<MovieLensRecommender>.Instance;
            using (IReleasableQueryable<User> users = recommender.Users)
            {
                User user = users.Single(element => element.UserId == userId);
                List<UserRating> userRatings = new List<UserRating>();
                if (null != user)
                {
                    using (IReleasableQueryable<UserAction> ratings = recommender.GetUserFeedback(user))
                    {
                        foreach (RatingAction rating in ratings.Where(element => element is RatingAction))
                        {
                            userRatings.Add(new UserRating
                            {
                                Title = rating.CatalogItem.ItemTitles.First().Title,
                                Rating = Convert.ToInt32(rating.ScaleRatingToApplication(1, 5))
                            });
                        }
                    }
                }
                else
                {
                    throw new ArgumentException("User not found.", "userId");
                }
                return userRatings.ToArray();
            }
        }

        /// <summary>
        /// Gets the ratings for user.
        /// </summary>
        /// <param name="userId">The user id.</param>
        /// <returns></returns>
        [OperationContract]
        public UserRating[] GetRatingsForUserByString(string userId)
        {
            return this.GetRatingsForUser(int.Parse(userId));
        }
    }
}
