﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Security.Cryptography.X509Certificates;
using MyMediaProject.RecommenderSystem.Core.Config;
using System.Web;
using EMIC.Design;
using System.Globalization;

namespace ExternalSystemIntegration
{
    /// <summary>
    /// Class providing useful static methods for various purposes
    /// </summary>
    public static class Utils
    {
        /// <summary>
        /// Gets or sets the proxy string.
        /// </summary>
        /// <value>The proxy string.</value>
        public static String ProxyString { get; set; }

        /// <summary>
        /// Calculated MD5 checksum from the given String
        /// </summary>
        /// <param name="valueToCalculateHashFrom">String to calculate MD5 hash from</param>
        /// <returns></returns>
        public static String Md5(String valueToCalculateHashFrom)
        {
            System.Security.Cryptography.MD5CryptoServiceProvider x = new System.Security.Cryptography.MD5CryptoServiceProvider();
            byte[] data = System.Text.Encoding.ASCII.GetBytes(valueToCalculateHashFrom);
            data = x.ComputeHash(data);
            String ret = "";
            for (int i = 0; i < data.Length; i++)
            {
                ret += data[i].ToString("x2", CultureInfo.CurrentCulture);
            }
            return ret;
        }

        /// <summary>
        /// Performs HTTPS request and returns the body of the HTTPS response 
        /// </summary>
        /// <param name="url"></param>
        /// <returns></returns>
        public static String MakeHttpsRequest(Uri url)
        {
            // convenient to configure proxy in the configuration once
            String proxyString = ConfigManager.ProxyString;

            return MakeHttpsRequest(url, proxyString);
        }


        /// <summary>
        /// Performs HTTPS request and returns the body of the HTTPS response
        /// </summary>
        /// <param name="url">The URL.</param>
        /// <param name="proxyString">Proxy string like 'http://my.proxy.com:8080'</param>
        /// <returns>The result(body) of the HTTPS response</returns>
        public static String MakeHttpsRequest(Uri url, String proxyString)
        {
            String result;

            // TraceLog must have strings containing curly braces passed as a parameter value (otherwise it fails)
            TraceLog.Write(LogEventType.Information, @"Making request to: {0}", new object[]{url});

            HttpWebRequest webRequest = (HttpWebRequest)WebRequest.Create(url);
            if (!String.IsNullOrEmpty(proxyString))
            {
                WebProxy proxy = new WebProxy();
                proxy.Address = new Uri(proxyString);
                webRequest.Proxy = proxy;
            }

            try
            {
                HttpWebResponse webResponse = (HttpWebResponse)webRequest.GetResponse();
                StreamReader streamReader = new StreamReader(webResponse.GetResponseStream(), Encoding.ASCII);
                result = streamReader.ReadToEnd();
                TraceLog.Write(LogEventType.Verbose, "HTTP Response: '{0}'.",  new object[]{result});
            }
            catch (Exception ex)
            {
                TraceLog.Write(LogEventType.Error, "Error making HTTP request : '{0}'", new object[] { ex.ToString() });
                throw;
            }
            return result;
        }


        /// <summary>
        /// Joins the given String-based Dictionary with URL like syntax
        /// </summary>
        /// <param name="stringDictionary">String-based Dictionary</param>
        /// <returns>joined key-values in the URL-like syntax</returns>
        public static String JoinDictionary(IDictionary<String, String> stringDictionary)
        {
            String result = "";
            foreach (String key in stringDictionary.Keys)
            {
                if (result.Length > 0)
                {
                    result += "&";
                }
                result += key + "=" + stringDictionary[key];
            }
            return result;
        }

        /// <summary>
        /// Joins the given String-based Dictionary with URL like syntax
        /// </summary>
        /// <param name="dictionaryParameters"></param>
        /// <param name="isUrlEncodingRequired"></param>
        /// <returns>joined key-values in the URL-like syntax</returns>
        public static String JoinDictionary(IDictionary<String, String> dictionaryParameters, bool isUrlEncodingRequired)
        {
            String result = "";
            foreach (String key in dictionaryParameters.Keys)
            {
                if (result.Length > 0)
                {
                    result += "&";
                }
                String value = dictionaryParameters[key];
                if (isUrlEncodingRequired)
                {
                    value = HttpUtility.UrlEncode(value);
                }
                result += key + "=" + value;
            }
            return result;
        }

        /// <summary>
        /// Joins String array values with a given separator
        /// </summary>
        /// <param name="array"></param>
        /// <param name="separator"></param>
        /// <returns>joined values with a given separator</returns>
        public static String JoinStringArray(String[] array, String separator)
        {
            String result = "";
            foreach (String elem in array)
            {
                if (result.Length > 0)
                {
                    result += separator;
                }
                result += elem;
            }
            return result;
        }

        /// <summary>
        /// Joins values (ToString() of each value) of the given array with a given separator
        /// </summary>
        /// <param name="array"></param>
        /// <param name="separator"></param>
        /// <returns></returns>
        public static String JoinArray(Array array, String separator)
        {
            String result = "";
            for (int i = 0; i < array.Length; i++)
            {
                if (result.Length > 0)
                {
                    result += separator;
                }
                if (array.GetValue(i) == null)
                {
                    continue;
                }
                result += array.GetValue(i).ToString();
            }
            return result;
        }


    }
}
